package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.dto.ApplyInfoDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * TODO
 * 注意事项：权限中心调用该接口后不能使用机构id,只能使用机构code
 *
 * @author yica
 * @date 2018/8/7
 */
@FeignClient(name = "dcuc-app", path = "/dcuc/app/applyInfoFacade")
public interface IApplyInfoFacade {

    /**
     * 分页查询应用
     *
     * @param searchDTO 查询条件
     * @return 应用分页
     */
    @PostMapping(value = "applyInfoPage")
    Page<ApplyInfo> applyInfoPage(@RequestBody SearchDTO searchDTO);


    /**
     * 详情
     *
     * @param id id
     * @return 应用信息
     */
    @RequestMapping(value = "applyDetail")
    ApplyInfo applyDetail(@RequestParam("id") String id);


    /**
     * 找到最新的排序
     *
     * @return 最新排序
     */
    @RequestMapping(value = "getNewSort")
    String getNewSort();

    /**
     * 启动或停止
     *
     * @param id 应用ID
     * @return 消息
     */
    @RequestMapping(value = "startOrStop")
    String startOrStop(@RequestParam("id") String id);

    /**
     * 排序应用
     *
     * @param id     源应用id
     * @param target 目标应用ID
     */
    @RequestMapping(value = "sortApp")
    void sortApp(@RequestParam("id") String id, @RequestParam("target") String target);

    /**
     * 根据appCode获取appId
     *
     * @param appCode 应用代码
     * @return 应用ID
     */
    @RequestMapping(value = "codeConvertToId")
    String codeConvertToId(@RequestParam("appCode") String appCode);

    /**
     * ID转换名称
     *
     * @param projectIds id
     * @return 名称
     */
    @PostMapping(value = "convertIdsToNames")
    String convertIdsToNames(@RequestBody String projectIds);

    /**
     * 根据appCode获取应用详情
     *
     * @param appCode 应用代码
     * @return 应用信息
     */
    @RequestMapping(value = "getAppByCode")
    ApplyInfo getAppByCode(@RequestParam("appCode") String appCode);

    /**
     * 是否重复
     *
     * @param applyInfo 应用信息
     * @return 是否重复
     */
    @PostMapping(value = "isRepeat")
    ResponseStatus isRepeat(@RequestBody ApplyInfo applyInfo);

//    /**
//     * 根据机构ID获取所有应用ID
//     *
//     * @param orgId 机构id
//     * @return 应用ID集合
//     */
//    @RequestMapping(value = "getAppIdsByOrgId")
//    List<String> getAppIdsByOrgId(@RequestParam("orgId") String orgId);

    /**
     * 根据施工人员id查看该人员负责的应用
     *
     * @param userId 用户id
     * @return 应用列表
     */
    @RequestMapping(value = "fzxmList")
    List<ApplyInfo> fzxmList(@RequestParam("userId") String userId);

    /**
     * 保存应用负责人
     *
     * @param dutyProject 应用ID
     * @param userId      用户id
     * @param mobileWork  工作手机号
     */
    @RequestMapping(value = "saveDutyMan")
    void saveDutyMan(@RequestParam("dutyProject") String dutyProject, @RequestParam("userId") String userId,
                     @RequestParam("mobileWork") String mobileWork);

    /**
     * 查询应用名称
     *
     * @return 应用名称和ID
     */
    @RequestMapping(value = "findAppNameMap")
    Map<String, String> findAppNameMap();

    /**
     * 人员授权-角色视图-查找应用列表
     *
     * @param searchDTO 搜索条件
     * @return 列表
     */
    @PostMapping(value = "findApplyList")
    Page<ApplyInfo> findApplyList(@RequestBody SearchDTO searchDTO);

    /**
     * 获取厂商所属应用个数
     *
     * @param manufacturerId 厂商ID
     * @return 个数
     */
    @RequestMapping(value = "countByManufacturerId")
    Integer countByManufacturerId(@RequestParam("manufacturerId") String manufacturerId);

    /**
     * 获取所有应用列表
     *
     * @return 应用列表
     */
    @RequestMapping(value = "getAllList")
    List<ApplyInfo> getAllList();

    /**
     * 根据厂商ID 获取应用信息
     *
     * @param manufacturerId 厂商ID
     * @return 应用列表
     */
    @RequestMapping("getAppListByManufacturerId")
    List<ApplyInfo> getAppListByManufacturerId(@RequestParam("manufacturerId") String manufacturerId);

    /**
     * 获取所有应用 map
     *
     * @return map
     */
    @RequestMapping("getAllIdNameMap")
    Map<String, String> getAllIdNameMap();

    /**
     * 根据应用代码、应用名称获取应用信息
     *
     * @param applyCode 应用代码
     * @param applyName 应用名称
     * @return 应用
     */
    @RequestMapping("getOneByCodeAndName")
    ApplyInfo getOneByCodeAndName(@RequestParam("applyCode") String applyCode, @RequestParam("applyName") String applyName);

    /**
     * 根据名称获取应用信息
     *
     * @param applyName 应用名称
     * @return 应用信息
     */
    @RequestMapping("getOneByName")
    ApplyInfo getOneByName(@RequestParam("applyName") String applyName);

    /**
     * 根据名称、厂商ID获取应用信息
     *
     * @param applyName      应用名称
     * @param manufacturerId 厂商ID
     * @return 应用信息
     */
    @RequestMapping("getOneByNameAndManufacturerId")
    ApplyInfo getOneByNameAndManufacturerId(@RequestParam("applyName") String applyName,
                                            @RequestParam("manufacturerId") String manufacturerId);


    /**
     * 根据id找到所有应用
     *
     * @param ids ids
     * @return 应用列表
     */
    @PostMapping("getAppById")
    List<ApplyInfo> getAppById(@RequestBody List<String> ids);

    /**
     * 根据应用名称LIKE获取应用列表
     *
     * @param applyName 应用名称
     * @return 应用列表
     */
    @PostMapping("getAppByAppNameLike")
    List<ApplyInfo> getAppByAppNameLike(@RequestParam("applyName") String applyName);

    /**
     * 保存
     *
     * @param applyInfoDTO 应用信息
     * @return 应用信息
     */
    @PostMapping(value = "saveAppIy")
    ApplyInfo saveAppIy(@RequestBody ApplyInfoDTO applyInfoDTO);

    /**
     * 修改
     *
     * @param applyInfoDTO 应用西悉尼
     * @return 应用信息
     */
    @PostMapping(value = "updateApply")
    ApplyInfo updateApply(@RequestBody ApplyInfoDTO applyInfoDTO);

    /**
     * 删除
     *
     * @param id 应用ID
     */
    @RequestMapping(value = "del")
    void del(@RequestParam("id") String id);

    /**
     * 根据应用URL获取应用信息
     *
     * @param applyUrl
     * @return
     */
    @RequestMapping(value = "getByApplyUrl")
    ApplyInfo getByApplyUrl(@RequestParam("applyUrl") String applyUrl);

    /**
     * 应用导入
     *
     * @param fileId
     * @return
     */
    @PostMapping(value = "/import")
    ResponseStatus impExcel(@RequestParam("fileId") String fileId);

    /**
     * 根据查询条件查询应用
     *
     * @param applyStatus 应用状态
     * @return
     */
    @GetMapping(value = "findList")
    List<ApplyInfo> findList(@RequestParam(value = "applyStatus", required = false) String applyStatus);

    /**
     * 根据code集合获取应用列表
     *
     * @param appCodes
     * @return
     */
    @PostMapping(value = "getListByCodes")
    List<ApplyInfo> getListByCodes(@RequestBody List<String> appCodes);

    /**
     * 条件查询-不分页
     *
     * @param searchDTO
     * @return
     */
    @PostMapping("appInfoSearch")
    List<ApplyInfo> appInfoSearch(@RequestBody SearchDTO searchDTO);

    /**
     * 更新机构信息
     *
     * @param targetOrgCode 目标机构代码
     * @param fullName      完整名称
     * @param code          机构代码
     * @return 状态
     */
    @PutMapping("updateOrgInfos")
    ResponseStatus updateOrgInfos(@RequestParam("targetOrgCode") String targetOrgCode, @RequestParam("fullName") String fullName,
                                  @RequestParam("code") String code);
}
