package com.dragoninfo.dcuc.app.facade;


import com.dragoninfo.dcuc.app.dto.*;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * @author mazq
 * @Description
 * @create 2020-11-25 14:21
 */
@FeignClient(name = "dcuc-app", path = "/dcuc/app/resourceFacade")
public interface IResourceFacade {
    /**
     * 保存资源数据到同步表中
     *
     * @param dto
     * @return
     */
    @PostMapping(value = "insertResourceSync")
    ResourceSyncDTO insertResourceSync(@RequestBody ResourceSyncDTO dto);

    /**
     * 批量保存资源数据到同步表
     *
     * @param list
     */
    @PostMapping(value = "batchInsertResourceSync")
    void batchInsertResourceSync(@RequestBody List<ResourceSyncDTO> list);

    /**
     * 写入应用资源
     *
     * @param dto
     * @return
     */
    @PostMapping(value = "insertAppResource")
    ApplyInfo insertAppResource(@RequestBody AppResourceDTO dto);

    /**
     * 写入服务资源
     *
     * @param dto
     * @return
     */
    @PostMapping(value = "insertServiceResource")
    ServiceResourceDTO insertServiceResource(@RequestBody ServiceResourceDTO dto);

    /**
     * 批量保存应用、服务资源
     *
     * @param list
     * @return
     */
    @PostMapping(value = "batchInsertAppResource")
    List<ApplyInfo> batchInsertAppResource(@RequestBody List<AppResourceDTO> list);

    /**
     * 批量保存应用、服务资源
     *
     * @param list
     * @return
     */
    @PostMapping(value = "batchInsertServiceResource")
    List<ServiceResource> batchInsertServiceResource(@RequestBody List<ServiceResourceDTO> list);


    /**
     * 资源同步表同步到应用表、服务表
     * @return
     */
    @PostMapping(value = "resourceSync")
    Boolean resourceSync();

    /**
     * 服务资源列表查询
     *
     * @param searchDTO
     * @return
     */
    @PostMapping(value = "serviceResourcePage")
    Page<ServiceResourceDTO> serviceResourcePage(@RequestBody SearchDTO searchDTO);
    /**
     * 从总线获取应用、服务、菜单资源
     * @param requestParam
     * @return
     */
    @PostMapping(value = "getResourceRemote")
    List<ResourceRequestResultDTO> getResourceRequestResults(@RequestBody ResourceRequestParamDTO requestParam);

    /**
     * 主客体授权-服务数据同步
     * @return
     */
    @GetMapping(value = "subSerSync")
    ResponseStatus subSerSync();

    /**
     * 主客体授权-应用和功能数据同步
     * @return
     */
    @GetMapping(value = "subAppSync")
    ResponseStatus subAppSync();
}
