package com.dragoninfo.dcuc.app.facade.sub;

import com.dragoninfo.dcuc.app.dto.sub.DataClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.DataClaDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 数据资源分类Api
 * @author mazq
 * @date 2021/5/25
 */
@FeignClient(name = "dcuc-app", path = "/dcuc/app/dataClaFacade")
public interface IDataClaFacade {

    /**
     * 四级资源分类保存
     * @param dto
     * @return
     */
    @PostMapping("dataClassifyAdd")
    ResponseStatus dataClassifyAdd(@RequestBody DataClaAcceptDTO dto);

    /**
     * 数据资源分类详情
     * @param id 数据资源分类 任何一级的id
     * @return
     */
    @GetMapping("detailAllLevel/{id}")
    DataClaAcceptDTO detailAllLevel(@PathVariable("id") String id);

    /**
     * 根据code和level查询单个层级详情
     * @param code
     * @param path
     * @return
     */
    @GetMapping("getByCodeAndPath")
    DataClaDTO getByCodeAndPath(@RequestParam("code") String code, @RequestParam(value = "path", required = false) String path);

    /**
     * 根据Id删除所有层级数据
     * @param id
     * @return
     */
    @DeleteMapping("deleteById/{id}")
    Boolean deleteAllLevel(@PathVariable("id") String id);

    /**
     * 四级资源分类保存
     * @param dto
     * @return
     */
    @PostMapping("dataClassifyUpdate")
    Boolean dataClassifyUpdate(@RequestBody DataClaAcceptDTO dto);

    /**
     * 数据资源分类分页列表(只查询第四级,拼接前三级字段)
     * @param searchDTO
     * @return
     */
    @PostMapping("pageSearch")
    Page<DataClaAcceptDTO> pageSearch(@RequestBody SearchDTO searchDTO);

    /**
     * 获取该层级数据分类列表
     * @param upId
     * @param level
     * @return
     */
    @GetMapping("getLevelList")
    List<DataClaDTO> getLevelList(@RequestParam(value = "upId", required = false) String upId, @RequestParam("level") String level);
}
