package com.dragoninfo.dcuc.app.facade.sub;

import com.dragoninfo.dcuc.app.dto.sub.FieldClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.FieldClaDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 字段分类表Api
 * @author mazq
 * @date 2021/5/25
 */
@FeignClient(name = "dcuc-app", path = "/dcuc/app/fieldClaFacade")
public interface IFieldClaFacade {

    /**
     * 两级字段分类保存
     * @param dto
     * @return
     */
    @PostMapping("fieldClassifyAdd")
    ResponseStatus fieldClassifyAdd(@RequestBody FieldClaAcceptDTO dto);

    /**
     * 两级字段分类修改
     * @param dto
     * @return
     */
    @PostMapping("fieldClassifyUpdate")
    Boolean fieldClassifyUpdate(@RequestBody FieldClaAcceptDTO dto);

    /**
     * 字段分类详情
     * @param id  字段分类的id
     * @return
     */
    @GetMapping("detailAllLevel/{id}")
    FieldClaAcceptDTO detailAllLevel(@PathVariable("id") String id);

    /**
     * 根据code和path查询字段分类详情
     * @param code
     * @param path
     * @return
     */
    @GetMapping("getByCodeAndPath")
    FieldClaDTO getByCodeAndPath(@RequestParam("code") String code, @RequestParam(value = "path", required = false) String path);

    /**
     * 根据Id删除所有层级数据
     * @param id
     * @return
     */
    @DeleteMapping("deleteById/{id}")
    Boolean deleteAllLevel(@PathVariable("id") String id);

    /**
     * 获取该层级字段分类列表
     * @param upId
     * @param level
     * @return
     */
    @GetMapping("getLevelList")
    List<FieldClaDTO> getLevelList(@RequestParam(value = "upId", required = false) String upId, @RequestParam("level") String level);

    /**
     * 两个层级字段分类分页列表查询
     * @param searchDTO
     * @return
     */
    @PostMapping("pageSearch")
    Page<FieldClaAcceptDTO> pageSearch(@RequestBody SearchDTO searchDTO);

}
