package com.dragoninfo.dcuc.app.business;

import com.dragoninfo.dcuc.app.dto.ResourceRequestParamDTO;
import com.dragoninfo.dcuc.app.dto.ResourceRequestResultDTO;

import java.lang.reflect.Type;
import java.util.List;
import java.util.Map;

/**
 * @author mazq
 * @date 2021/2/26
 */
public interface IResourceBusiness {


    /**
     * 从总线获取应用、服务、菜单资源
     *
     * @param requestParamDTO
     * @return
     */
    List<ResourceRequestResultDTO> getResourceRequestResults(ResourceRequestParamDTO requestParamDTO);

    /**
     * 资源同步到应用和服务表
     * @return 同步结果
     */
    Boolean resourceSync();

    /**
     * 应用资源增量同步
     * 只用于应用消息同步变更
     *
     * @param appCode
     */
    boolean appIncrSync(String appCode);

    /**
     * 服务资源同步
     *
     * @param serviceCode
     * @return
     */
    boolean serviceIncrSync(String serviceCode);

    /**
     * 应用启用
     * 只用于应用消息同步变更
     *
     * @param appCode
     * @return
     */
    boolean startApp(String appCode);

    /**
     * 应用停用
     * 只用于应用消息同步变更
     *
     * @param appCode
     * @return
     */
    boolean stopApp(String appCode);

    /**
     * 应用注销
     * 只用于应用消息同步变更
     *
     * @param appCode
     * @return
     */
    boolean cancelApp(String appCode);

    /**
     * 服务启用
     * 只用于应用消息同步变更
     *
     * @param serviceCode
     * @return
     */
    boolean startService(String serviceCode);

    /**
     * 服务停用
     * 只用于应用消息同步变更
     *
     * @param serviceCode
     * @return
     */
    boolean stopService(String serviceCode);

    /**
     * 服务注销
     * 只用于应用消息同步变更
     *
     * @param serviceCode
     * @return
     */
    boolean cancelService(String serviceCode);

    <T> T getDataResource(Map paramMap, Type type);

}
