package com.dragoninfo.dcuc.app.business.impl;

import cn.hutool.core.thread.ThreadFactoryBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.app.business.IResourceBusiness;
import com.dragoninfo.dcuc.app.config.DcucResourceConfig;
import com.dragoninfo.dcuc.app.cons.ServiceStatusCons;
import com.dragoninfo.dcuc.app.dto.*;
import com.dragoninfo.dcuc.app.dto.dataresource.BaseResp;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ResourceSync;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.enumresources.ResourceTypeEnum;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragoninfo.dcuc.app.service.IResourceService;
import com.dragoninfo.dcuc.app.service.IResourceSyncService;
import com.dragoninfo.dcuc.app.service.IServiceResourceService;
import com.dragoninfo.dcuc.app.service.impl.ResourceServiceImpl;
import com.dragoninfo.dcuc.app.vo.BusSendReqVO;
import com.dragoninfo.dcuc.auth.auth.facade.IBimBusinessFacade;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.http.HttpUtil;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.ObjectUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.google.common.reflect.TypeToken;
import org.apache.http.Header;
import org.apache.http.message.BasicHeader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/2/26
 */
@Component
public class ResourceBusiness implements IResourceBusiness {

    Logger logger = LoggerFactory.getLogger(ResourceBusiness.class);

    @Autowired
    private IResourceService resourceService;
    @Autowired
    private IResourceSyncService syncService;
    @Autowired
    private DcucResourceConfig resourceConfig;
    @Autowired
    private IApplyInfoService applyInfoService;
    @Autowired
    private IServiceResourceService serviceResourceService;
    @Autowired
    private IBimBusinessFacade bimBusinessFacade;
    /**
     * 使用线程池异步获取应用菜单
     */
    private ExecutorService executor = new ThreadPoolExecutor(1, 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(),
            new ThreadFactoryBuilder().setNamePrefix("menu-resource").build());
    /**
     * 重试次数
     */
    public static final int RETRY_NUMBER = 3;

    @Override
    public Boolean resourceSync() {
        //先同步应用和服务数据
        //再同步应用菜单数据
        String[] resourceIds = new String[]{ResourceTypeEnum.TJ_APP.getResourceId(),
                ResourceTypeEnum.TJ_SERVICE.getResourceId(),
                ResourceTypeEnum.TJ_APP_MENU.getResourceId(),
                ResourceTypeEnum.BU_SERVICE.getResourceId()};
        List<ResourceSync> resourceSyncs;
        for (String resourceId : resourceIds) {
            resourceSyncs = syncService.getNotSyncResource(resourceId);
            if (CollectionUtils.isEmpty(resourceSyncs)) {
                continue;
            }
            resourceService.resourceSync(resourceSyncs);
            //更新sync字段
            resourceSyncs.forEach(item -> item.setSync(BooleanEnum.TRUE.getValue()));
            syncService.batchUpdate(resourceSyncs);
        }
        return true;
    }

    @Override
    public boolean appIncrSync(String appCode) {
        //调用总线接口根据appCode获取应用详情
        AppResourceAcceptDTO appMenuDTO;
        String appStr = null;
        String userToken = bimBusinessFacade.getUserToken(false);

        String appToken = bimBusinessFacade.getAppToken(userToken, false);
        try {
            logger.info("-------getAppRemoteDetailByCode STATR---------------");
            appStr = getAppRemoteDetailByCode(appCode, userToken, appToken, 0);
            logger.info("-------getAppRemoteDetailByCode END---------------");
            appMenuDTO = JSON.parseObject(appStr, AppResourceAcceptDTO.class);
        } catch (Exception e) {
            logger.info("get app info from remote appStr:{}", appStr);
            logger.error("get app info from remote error.", e);
            return false;
        }
        if (null == appMenuDTO) {
            logger.info("appIncrSync get appInfo is null. appCode is:{}", appCode);
            return false;
        }
        acceptAppResource(appStr, appMenuDTO);
        return false;
    }

    @Override
    public boolean serviceIncrSync(String serviceCode) {
        ServiceResourceAcceptDTO serviceDTO;
        String serviceStr = null;
        try {
            logger.info("-------getServiceRemoteDetailByCode STATR---------------");
            serviceStr = getServiceRemoteDetailByCode(serviceCode);
            logger.info("-------getServiceRemoteDetailByCode END---------------");
            serviceDTO = JSON.parseObject(serviceStr, ServiceResourceAcceptDTO.class);
        } catch (Exception e) {
            logger.info("get service info from remote serviceStr:{}", serviceStr);
            logger.error("get service info from remote error.", e);
            return false;
        }
        if (null == serviceDTO) {
            logger.info("appIncrSync get serviceInfo is null. serviceCode is:{}", serviceCode);
            return false;
        }
        acceptServiceResource(serviceDTO);
        return true;
    }

    @Override
    public boolean startApp(String appCode) {
        ApplyInfo applyInfo = applyInfoService.getAppByCode(appCode);
        if (null == applyInfo) {
            return false;
        }
        ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
        BeanUtils.copyProperties(applyInfo, applyInfoDTO);
        //启用是0,停用是1
        applyInfoDTO.setApplyStatus(BooleanEnum.FALSE.getValue());
        applyInfoService.updateApply(applyInfoDTO);
        return true;
    }

    @Override
    public boolean stopApp(String appCode) {
        ApplyInfo applyInfo = applyInfoService.getAppByCode(appCode);
        if (null == applyInfo) {
            return false;
        }
        ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
        BeanUtils.copyProperties(applyInfo, applyInfoDTO);
        //启用是0,停用是1
        applyInfoDTO.setApplyStatus(BooleanEnum.TRUE.getValue());
        applyInfoService.updateApply(applyInfoDTO);
        return true;
    }

    @Override
    public boolean cancelApp(String appCode) {
        ApplyInfo applyInfo = applyInfoService.getAppByCode(appCode);
        if (null == applyInfo) {
            return false;
        }
        applyInfoService.del(applyInfo.getId());
        return true;
    }

    @Override
    public boolean startService(String serviceCode) {
        ServiceResource serviceResource = serviceResourceService.detailByCode(serviceCode);
        if (null == serviceResource) {
            return false;
        }
        ServiceResourceDTO dto = new ServiceResourceDTO();
        BeanUtils.copyProperties(serviceResource, dto);
        dto.setServiceStatus(BooleanEnum.TRUE.getValue());
        serviceResourceService.updateById(dto);
        return true;
    }

    @Override
    public boolean stopService(String serviceCode) {
        ServiceResource serviceResource = serviceResourceService.detailByCode(serviceCode);
        if (null == serviceResource) {
            return false;
        }
        ServiceResourceDTO dto = new ServiceResourceDTO();
        BeanUtils.copyProperties(serviceResource, dto);
        dto.setServiceStatus(BooleanEnum.FALSE.getValue());
        serviceResourceService.updateById(dto);
        return false;
    }

    @Override
    public boolean cancelService(String serviceCode) {
        ServiceResource serviceResource = serviceResourceService.detailByCode(serviceCode);
        if (null == serviceResource) {
            return false;
        }
        ServiceResourceDTO dto = new ServiceResourceDTO();
        BeanUtils.copyProperties(serviceResource, dto);
        dto.setServiceStatus(ServiceStatusCons.STATUS_DELETED);
        dto.setDeleted(BooleanEnum.TRUE.getValue());
        serviceResourceService.updateById(dto);
        serviceResourceService.deleteById(dto.getId());
        return true;
    }

    @Override
    public <T> T getDataResource(Map paramMap, Type type) {
        // 构建header
        String busSraId = resourceConfig.getBusSraId();

        String busSreIdcard = UserContextUtils.getCurrentUser().getIdcard();

        String busServiceId = resourceConfig.getDataResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getDataResource().getBusServiceFunc();

        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(busSreIdcard)
                .build();

        List<Header> headerList = getHeaders(busSendReqVO);
        String url = resourceConfig.getResourceUrl();
        logger.info("getDataResource >> url:{}, busSraId:{}, busServiceId:{}, busServiceFunc:{}, busSreIdcard:{}",
                url, busSraId, busServiceId, busServiceFunc, busSreIdcard);
        String response = HttpUtil.postJSON(url, JSON.toJSONString(paramMap), headerList, null);
        logger.info("getDataResource response:{}", response);
        BaseResp<String> baseResp = JSON.parseObject(response, new TypeToken<BaseResp<String>>() {
        }.getType());
        if ("000".equals(baseResp.getCode())) {
            return JSON.parseObject(baseResp.getResultData(), type);
        }
        logger.error("调用数据失败：getDataResource >> url:{}, busSraId:{}, busServiceId:{}, busServiceFunc:{}, busSreIdcard:{}",
                url, busSraId, busServiceId, busServiceFunc, busSreIdcard);
        return null;
    }

    private String getServiceRemoteDetailByCode(String serviceCode) {

        String userToken = bimBusinessFacade.getUserToken(false);

        String appToken = bimBusinessFacade.getAppToken(userToken, false);

        String response = getAppResource(serviceCode, userToken, appToken, 0);

        logger.info("getServiceRemoteDetailByCode result:{}", response);
        if (StringUtils.isBlank(response)) {
            logger.info("getServiceRemoteDetailByCode response is null");
            return "";
        }

        JSONObject jsonObject = JSONObject.parseObject(response);
        String status = jsonObject.getString("status");
        if (!ResponseStatus.SUCCESS_CODE.equals(status)) {
            logger.info("getServiceRemoteDetailByCode status code:{}", status);
            return "";
        }
        JSONObject o = jsonObject.getJSONObject("data").getJSONArray("Resources").getJSONObject(0);
        if (ObjectUtils.isEmpty(o)) {
            logger.info("getServiceRemoteDetailByCode Resources is null");
            return "";
        }
        JSONObject dataInfo = o.getJSONObject("DataInfo");
        if (ObjectUtils.isEmpty(dataInfo)) {
            logger.info("getServiceRemoteDetailByCode dataInfo is null");
            return "";
        }
        return JSON.toJSONString(dataInfo);
    }

    /**
     * 获取应用资源
     *
     * @param serviceCode 服务代码
     * @param userToken   用户令牌
     * @param appToken    应用令牌
     * @param retry       重试次数
     * @return 返回结果
     */
    private String getAppResource(String serviceCode, String userToken, String appToken, int retry) {
        String busSraId = resourceConfig.getBusSraId();
        String busSreIdcard = resourceConfig.getBusSreIdcard();
        String busServiceId = resourceConfig.getServiceModifyResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getServiceModifyResource().getBusServiceFunc();

        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(busSreIdcard)
                .userToken(userToken)
                .appToken(appToken)
                .build();

        List<Header> headerList = getHeaders(busSendReqVO);

        String url = resourceConfig.getResourceUrl();
        Map<String, String> param = new HashMap<>();
        param.put("Condition", "FWZYBSF = '" + serviceCode + "'");
        param.put("page", "1");
        param.put("pageSize", "1");

        logger.info("getServiceRemoteDetailByCode >> url:{}, busSraId:{}, busServiceId:{}, busServiceFunc:{}, busSreIdcard:{}, serviceCode:{}",
                url, busSraId, busServiceId, busServiceFunc, busSreIdcard, serviceCode);

        String response = HttpUtil.postJSON(url, JSON.toJSONString(param), headerList, null);

        if ((retry <= RETRY_NUMBER) && (!JSONUtil.isJson(response))) {
            retry++;
            String retryUserToken = bimBusinessFacade.getUserToken(true);
            String retryAppToken = bimBusinessFacade.getAppToken(userToken, true);
            return getAppResource(serviceCode, retryUserToken, retryAppToken, retry);
        }

        return response;
    }

    /**
     * 增量新增/更新应用资源和应用相关的菜单资源
     *
     * @param appStr
     * @param acceptDTO
     * @return
     */
    public boolean acceptAppResource(String appStr, AppResourceAcceptDTO acceptDTO) {
        AppStrAcceptDTO strAcceptDTO = JSON.parseObject(appStr, AppStrAcceptDTO.class);

        ResourceTypeEnum appEnum = ResourceTypeEnum.TJ_APP;
        ResourceSyncDTO syncDTO = new ResourceSyncDTO(appEnum.getResourceId()
                , appEnum.getResourceType(), appEnum.getResourceProvider());
        AppInfoAcceptDTO appAcceptDTO = acceptDTO.getTappSystem();

        syncDTO.setAppCode(appAcceptDTO.getYyxtbh());
        syncDTO.setAppName(appAcceptDTO.getYyxtmc());
        syncDTO.setExtId(appAcceptDTO.getId());
        syncDTO.setContent(strAcceptDTO.getTappSystem());
        syncDTO.setDeleted(BooleanEnum.FALSE.getValue());
        syncDTO.setSync(BooleanEnum.TRUE.getValue());
        syncService.insertRemoteResourceSync(syncDTO);

        ResourceTypeEnum menuEnum = ResourceTypeEnum.TJ_APP_MENU;
        ResourceSyncDTO menuSyncDTO = new ResourceSyncDTO(menuEnum.getResourceId()
                , menuEnum.getResourceType(), menuEnum.getResourceProvider());
        menuSyncDTO.setAppCode(appAcceptDTO.getYyxtbh());
        menuSyncDTO.setAppName(appAcceptDTO.getYyxtmc());
        menuSyncDTO.setExtId(appAcceptDTO.getId());
        menuSyncDTO.setContent(strAcceptDTO.getTappFunctionList());
        menuSyncDTO.setDeleted(BooleanEnum.FALSE.getValue());
        menuSyncDTO.setSync(BooleanEnum.TRUE.getValue());
        syncService.insertRemoteResourceSync(menuSyncDTO);

        //同步更新应用表
        AppResourceDTO appResourceDTO = convertToAppDTO(acceptDTO);
        resourceService.insertAppResource(appResourceDTO);
        return true;
    }

    /**
     * 应用资源接收类转为应用DTO
     *
     * @param acceptDTO
     * @return
     */
    private AppResourceDTO convertToAppDTO(AppResourceAcceptDTO acceptDTO) {
        AppInfoAcceptDTO appInfoAcceptDTO = acceptDTO.getTappSystem();
        AppResourceDTO dto = new AppResourceDTO();
        dto.setDeleted(appInfoAcceptDTO.getXtzxbs());
        dto.setAppCode(appInfoAcceptDTO.getYyxtbh());
        dto.setAppName(appInfoAcceptDTO.getYyxtmc());
        String xtzybs = appInfoAcceptDTO.getXtzybs();
        if (StrUtil.isNotBlank(xtzybs)) {
            if (BooleanEnum.TRUE.getValue().equals(xtzybs)) {
                xtzybs = BooleanEnum.FALSE.getValue();
            } else {
                xtzybs = BooleanEnum.TRUE.getValue();
            }
        }
        dto.setAppStatus(xtzybs);
        dto.setAppUrl(appInfoAcceptDTO.getYyxtfwdz());
        dto.setBuildUnitCode(appInfoAcceptDTO.getYyxtsqdwdm());
        dto.setManagerUnitCode(appInfoAcceptDTO.getYyxtgldwdm());
        dto.setContractUnit(appInfoAcceptDTO.getYyxtcjdwmc());
        dto.setResourceId(ResourceTypeEnum.TJ_APP.getResourceId());
        dto.setResourceType(ResourceTypeEnum.TJ_APP.getResourceType());
        dto.setResourceProvider(ResourceTypeEnum.TJ_APP.getResourceProvider());
        String sxrq = appInfoAcceptDTO.getSxrq();
        if (StrUtil.isNotBlank(sxrq)) {
            dto.setLineTime(sxrq);
            try {
                Date date = DateUtils.getDate(sxrq, DateConst.HYPHEN_DISPLAY_TIME);
                dto.setRegistrationTime(date);
            } catch (ParseException e) {
                logger.error("Parse error", e);
            }
        }
        dto.setApplyCategory(appInfoAcceptDTO.getYyxtfldm());
        dto.setPoliceSpecies(appInfoAcceptDTO.getGajzfldm());
        dto.setManagerDescribe(appInfoAcceptDTO.getYyxtsm());
        AppResourceDTO appResourceDTO = ResourceServiceImpl.convertToAppDTO(appInfoAcceptDTO);
        //设置菜单资源
        List<MenuResourceAcceptDTO> tAppFunctionList = acceptDTO.getTappFunctionList();
        List<MenuResourceDTO> dtoArrayList = new ArrayList<>();
        for (MenuResourceAcceptDTO menuResourceAcceptDTO : tAppFunctionList) {
            MenuResourceDTO menuDTO = new MenuResourceDTO();
            menuDTO.setFunCode(menuResourceAcceptDTO.getGnbh());
            menuDTO.setPFunCode(menuResourceAcceptDTO.getSjgnbh());
            menuDTO.setFunName(menuResourceAcceptDTO.getGnmc());
            menuDTO.setAppCode(appInfoAcceptDTO.getYyxtbh());
            menuDTO.setStatus(menuResourceAcceptDTO.getZyzt());
            dtoArrayList.add(menuDTO);
        }
        appResourceDTO.setMenusResources(dtoArrayList);
        String appCode = appInfoAcceptDTO.getYyxtbh();
        List<MenuResourceDTO> menusDTOs = ResourceServiceImpl.convertToMenuDTOs(appCode, tAppFunctionList);
        appResourceDTO.setMenusResources(menusDTOs);
        return appResourceDTO;
    }

    /**
     * 增量新增服务资源
     *
     * @param acceptDTO
     * @return
     */
    public boolean acceptServiceResource(ServiceResourceAcceptDTO acceptDTO) {
        ResourceTypeEnum serviceEnum = ResourceTypeEnum.TJ_SERVICE;
        ResourceSyncDTO syncDTO = new ResourceSyncDTO(serviceEnum.getResourceId()
                , serviceEnum.getResourceType(), serviceEnum.getResourceProvider());
        syncDTO.setServiceCode(acceptDTO.getFWZYBSF());
        syncDTO.setServiceName(acceptDTO.getFWZYMC());
        syncDTO.setExtId(acceptDTO.getId());
        syncDTO.setContent(JSON.toJSONString(acceptDTO));
        syncDTO.setSync(BooleanEnum.TRUE.getValue());
        syncDTO.setDeleted(BooleanEnum.FALSE.getValue());
        syncService.insertRemoteResourceSync(syncDTO);
        //同步更新服务表
        ServiceResourceDTO serviceResourceDTO = convertToServiceDTO(acceptDTO);
        serviceResourceDTO.setResourceId(serviceEnum.getResourceId());
        serviceResourceDTO.setResourceType(serviceEnum.getResourceType());
        serviceResourceDTO.setResourceProvider(serviceEnum.getResourceProvider());
        resourceService.insertServiceResource(serviceResourceDTO);
        return true;
    }

    private ServiceResourceDTO convertToServiceDTO(ServiceResourceAcceptDTO acceptDTO) {
        ServiceResourceDTO dto = new ServiceResourceDTO();
        dto.setServiceCode(acceptDTO.getFWZYBSF());
        dto.setServiceName(acceptDTO.getFWZYMC());
        dto.setManagerUnitCode(acceptDTO.getFWZYGLDW_GAJGJGDM());
        dto.setBuildUnitCode(acceptDTO.getFWZYSQDW_JGDM());
        dto.setServiceUrl(acceptDTO.getFWFWDZ());
        String fwzyztdm = acceptDTO.getFWZYZTDM();
        dto.setServiceStatus(fwzyztdm);
        dto.setDeleted(BooleanEnum.FALSE.getValue());
        if (ServiceStatusCons.STATUS_DELETED.equals(fwzyztdm)) {
            dto.setDeleted(BooleanEnum.TRUE.getValue());
        }
        dto.setServiceType(acceptDTO.getFWLXDM());
        dto.setServiceProvideWay(acceptDTO.getFWTGFSDM());
        //是否自用服务，服务资源未返回
//        dto.setMustSelf();
        dto.setAppCode(acceptDTO.getTGFWZYD_YYZYBS());
        return dto;
    }


    @Override
    public List<ResourceRequestResultDTO> getResourceRequestResults(ResourceRequestParamDTO requestParamDTO) {
        List<ResourceRequestResultDTO> pageInfos = new ArrayList<>();
        Integer pageSize = requestParamDTO.getPageSize();
        Boolean listingAll = requestParamDTO.getListingAll();
        Boolean menuSync = requestParamDTO.getMenuSync();
        if (null == listingAll) {
            listingAll = true;
        }
        if (null == menuSync) {
            menuSync = false;
        }
        pageSize = (pageSize == null ? 50 : pageSize);
        //全量获取
        //从currentPageNo页获取
        if (listingAll) {
            //全量同步默认每页数量,便于下次同步
            pageSize = resourceConfig.getPageSize();
            Integer currentPageNo = requestParamDTO.getCurrentPageNo();
            List<String> resourceTypeList = requestParamDTO.getListAllResourceType();
            for (ResourceTypeEnum resourceEnum : ResourceTypeEnum.values()) {
                if (CollectionUtils.isNotEmpty(resourceTypeList)
                        && !resourceTypeList.contains(resourceEnum.getResourceType())) {
                    continue;
                }
                //若是增量获取currentPage = 数据库中上次同步的页数
                //暂时手动输入currentPage
                //没有就从第一页开始获获取
                if (null == currentPageNo) {
                    currentPageNo = 1;
                }
                ResourceRequestResultDTO listingInfo = getAllResourceListing(menuSync, currentPageNo, pageSize, resourceEnum);
                pageInfos.add(listingInfo);
            }
        } else {
            //根据页码获取
            List<ListingParamDTO> listingParams = requestParamDTO.getListingParams();
            for (ListingParamDTO listingParam : listingParams) {
                ResourceRequestResultDTO listingInfo = getPageNoResourceListing(menuSync, pageSize, listingParam);
                pageInfos.add(listingInfo);
            }
        }
        //是否需要保存已获取的页数,下次获取使用增量获取？
        return pageInfos;
    }


    /**
     * 根据页码获取资源列表
     *
     * @param menuSync
     * @param pageSize
     * @param listingParam
     * @return
     */
    private ResourceRequestResultDTO getPageNoResourceListing(Boolean menuSync, Integer pageSize, ListingParamDTO listingParam) {
        String resourceId = listingParam.getResourceId();
        List<Integer> pageNos = listingParam.getPageNo();
        ResourceRequestResultDTO resourceListResult = new ResourceRequestResultDTO(resourceId, new ArrayList<Integer>());
        resourceListResult.setSuccess(BooleanEnum.TRUE.getValue());
        for (Integer pageNo : pageNos) {
            Result<Map<String, Object>> result = getRemoteResource(menuSync, resourceId, pageNo, pageSize);
            //只收集未同步成功的资源id和页码
            if (!ResponseStatus.SUCCESS_CODE.equals(result.getResult())) {
                Map<String, Object> resultMap = result.getContent();
                Integer ePageNo = (Integer) resultMap.get("pageNo");
                resourceListResult.getPageNos().add(ePageNo);
                resourceListResult.setSuccess(BooleanEnum.FALSE.getValue());
            }
        }
        //不记录总页数,这里不是全量获取
        resourceListResult.setResourceId(resourceId);
        return resourceListResult;
    }

    /**
     * 从currentPageNo页获取全量的资源列表
     *
     * @param menuSync
     * @param pageSize
     * @return
     */
    private ResourceRequestResultDTO getAllResourceListing(Boolean menuSync, Integer currentPageNo, Integer pageSize, ResourceTypeEnum resourceTypeEnum) {
        Integer pageNo = currentPageNo;
        Integer pages = currentPageNo;
        String resourceId = resourceTypeEnum.getResourceId();
        ResourceRequestResultDTO resourceListResult = new ResourceRequestResultDTO(resourceId, new ArrayList<>());
        resourceListResult.setSuccess(BooleanEnum.TRUE.getValue());
        do {
            Result<Map<String, Object>> result = getRemoteResource(menuSync, resourceId, pageNo, pageSize);
            Map<String, Object> resultMap = result.getContent();
            //收集未同步成功的资源id和页码
            if (!ResponseStatus.SUCCESS_CODE.equals(result.getResult())) {
                Integer ePageNo = (Integer) resultMap.get("pageNo");
                resourceListResult.getPageNos().add(ePageNo);
                resourceListResult.setSuccess(BooleanEnum.FALSE.getValue());
            }
            Integer allPages = (Integer) resultMap.get("pages");
            if (null != allPages) {
                pages = allPages;
            }
            ++pageNo;
        } while (pageNo <= pages);
        //收集获取列表的信息
        //这里是全量获取,记录资源获取的总页数,便于下次全量获取使用
        resourceListResult.setTotalPages(pages);
        resourceListResult.setResourceId(resourceId);
        return resourceListResult;
    }

    private void getAppMenus(Boolean menuSync, List<String> extAppIds) {
        if (CollectionUtils.isEmpty(extAppIds)) {
            return;
        }
        String idcard = UserContextUtils.getCurrentUser().getIdcard();
        MenuTask menuTask = new MenuTask(extAppIds, idcard);
        if (!menuSync) {
            executor.submit(menuTask);
        } else {
            menuTask.run();
        }
    }

    /**
     * 从总线获取应用详细信息
     *
     * @param appId 资源提供方应用id
     * @return
     */
    private String getAppRemoteDetailById(String appId, String idcard) {
        String busSraId = resourceConfig.getBusSraId();
        String busServiceId = resourceConfig.getMenuResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getMenuResource().getBusServiceFunc();

        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(idcard)
                .build();

        List<Header> headerList = getHeaders(busSendReqVO);
        String url = resourceConfig.getResourceUrl() + "/" + appId;
        logger.info("getAppRemoteDetailById >> url:{}", url);
        String result = HttpUtil.get(url, null, headerList);
        if (StringUtils.isBlank(result)) {
            logger.info("getAppRemoteDetailById response is null");
            return "";
        }
        logger.info("getAppRemoteDetailById result:{}", result);
        return result;
    }

    /**
     * 从总线获取应用详细信息
     *
     * @param appCode   应用code
     * @param userToken 用户令牌
     * @param appToken  应用令牌
     * @return 结果
     */
    private String getAppRemoteDetailByCode(String appCode, String userToken, String appToken, int retry) {
        String busSraId = resourceConfig.getBusSraId();
        String busSreIdcard = resourceConfig.getBusSreIdcard();
        String busServiceId = resourceConfig.getAppModifyResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getAppModifyResource().getBusServiceFunc();
        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(busSreIdcard)
                .userToken(userToken)
                .appToken(appToken)
                .build();

        List<Header> headerList = getHeaders(busSendReqVO);

        String url = resourceConfig.getResourceUrl();
        Map<String, String> param = new HashMap<>();
        param.put("yyxtbh", appCode);
        logger.info("getAppRemoteDetailByCode >> url:{}, busSraId:{}, busServiceId:{}, busServiceFunc:{}, busSreIdcard:{}, appCode:{}",
                url, busSraId, busServiceId, busServiceFunc, busSreIdcard, appCode);
        String response = HttpUtil.get(url, param, headerList);
        if (StringUtils.isBlank(response)) {
            logger.info("getAppRemoteDetailByCode response is null");
            return "";
        }
        logger.info("getAppRemoteDetailByCode result:{}", response);

        if ((retry <= RETRY_NUMBER) && (!JSONUtil.isJson(response))) {
            retry++;
            String retryUserToken = bimBusinessFacade.getUserToken(true);
            String retryAppToken = bimBusinessFacade.getAppToken(userToken, true);
            return getAppRemoteDetailByCode(appCode, retryUserToken, retryAppToken, retry);
        }

        return response;
    }

    /**
     * 从总线分页查询服务资源
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    private HttpResult getServiceResource(Integer pageNo, Integer pageSize) {
        String url = resourceConfig.getResourceUrl();
        HttpResult httpResult = new HttpResult();
        Map<String, Object> map = new HashMap<>();
        map.put("page", String.valueOf(pageNo - 1));
        String param = JSON.toJSONString(map);
        //获取服务资源
        //服务资源提供方一次查询所有服务资源,未采用分页获取方式
        String busSraId = resourceConfig.getBusSraId();
        String busSreIdcard = UserContextUtils.getCurrentUser().getIdcard();
        String busServiceId = resourceConfig.getServiceResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getServiceResource().getBusServiceFunc();

        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(busSreIdcard)
                .build();
        List<Header> headerList = getHeaders(busSendReqVO);
        logger.info("-------------getServiceResource START -------------");
        logger.info("getServiceResource url:{}", url);
        String response = HttpUtil.postJSON(url, param, headerList, null);
        if (null == response) {
            logger.info("get service resource fail,pageNo:{},pageSize:{}", pageNo, pageSize);
            httpResult.setCode(ResponseStatus.FAIL_CODE);
        } else {
            logger.info("get service resource success,pageNo:{},pageSize:{}", pageNo, pageSize);
            JSONObject parse = JSONObject.parseObject(response);
            JSONObject data = parse.getJSONObject("data");
            httpResult.setRecords(data.getJSONArray("content"));
            httpResult.setCode(parse.getString("status"));
            httpResult.setPages(data.getInteger("totalPages"));
            httpResult.setTotals(data.getInteger("totalElements"));
        }
        logger.info("-------------getServiceResource END -------------");
        return httpResult;
    }


    /**
     * 从网关获取应用信息和服务信息,并录入数据
     */
    private Result<Map<String, Object>> getRemoteResource(Boolean menuSync, String resouceId, Integer pageNo, Integer pageSize) {
        Map<String, Object> resultMap = new HashMap<>();
        ResourceTypeEnum resourceEnum = ResourceTypeEnum.getByResourceId(resouceId);
        HttpResult resultFromRemote;
        try {
            resultFromRemote = getResourceFromRemote(resourceEnum, pageNo, pageSize);
        } catch (Exception e) {
            logger.info("get remote resource error from pageNo:{},pageSize:{},resourceId:{}", pageNo, pageSize, resouceId);
            logger.error("get remote resource error", e);
            resultMap.put("pageNo", pageNo);
            resultMap.put("pageSize", pageSize);
            resultMap.put("resourceId", resouceId);
            return Result.fail(ResponseStatus.FAIL_CODE, "get remote resource error", resultMap);
        }
        if (!ResponseStatus.SUCCESS_CODE.equals(resultFromRemote.getCode())) {
            logger.info("get remote resource error from pageNo:{},pageSize:{},resourceId:{}", pageNo, pageSize, resouceId);
            logger.info("get remote resource error:{}", JSON.toJSONString(resultFromRemote));
            resultMap.put("pageNo", pageNo);
            resultMap.put("pageSize", pageSize);
            resultMap.put("resourceId", resouceId);
            return Result.fail(ResponseStatus.FAIL_CODE, "get remote resource failed", resultMap);
        }
        JSONArray records = resultFromRemote.getRecords();
        //插入资源数据同步表
        if (CollectionUtils.isNotEmpty(records)) {
            List<ResourceSyncDTO> dtos = insertRemoteResourceSync(resourceEnum, records);
            if (resourceConfig.getMenuResource().isMenuEnabled() && ResourceTypeEnum.TJ_APP.equals(resourceEnum)) {
                List<String> ids = dtos.stream()
                        .filter(item -> StringUtils.isNotBlank(item.getExtId()))
                        .map(ResourceSyncDTO::getExtId)
                        .collect(Collectors.toList());
                getAppMenus(menuSync, ids);
            }
        }
        //返回总数量和总页数
        Integer total = resultFromRemote.getTotals();
        Integer pages = resultFromRemote.getPages();
        resultMap.put("pageNo", pageNo);
        resultMap.put("pageSize", pageSize);
        resultMap.put("resourceId", resouceId);
        resultMap.put("total", total);
        resultMap.put("pages", pages);
        return Result.success(resultMap);
    }

    /**
     * 获取应用信息
     *
     * @return
     */
    private HttpResult getResourceFromRemote(ResourceTypeEnum resourceTypeEnum, Integer pageNo, Integer pageSize) {
        HttpResult httpResult;
        //获取应用资源
        if (ResourceTypeEnum.TJ_APP.equals(resourceTypeEnum)) {
            httpResult = getAppResource(pageNo, pageSize);
        } else if (ResourceTypeEnum.TJ_SERVICE.equals(resourceTypeEnum)) {
            httpResult = getServiceResource(pageNo, pageSize);
        } else if (ResourceTypeEnum.BU_SERVICE.equals(resourceTypeEnum)) {
            httpResult = getBuServiceResource(pageNo, pageSize);
        } else {
            httpResult = new HttpResult();
            httpResult.setCode(ResponseStatus.SUCCESS_CODE);
            httpResult.setPages(pageNo);
            httpResult.setTotals(0);
        }
        return httpResult;
    }

    /**
     * 从大数据平台获取
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    private HttpResult getBuServiceResource(Integer pageNo, Integer pageSize) {
        String resourceUrl = resourceConfig.getBuServiceUrl();
        HttpResult httpResult = new HttpResult();
        Map<String, Object> map = new HashMap<>();
        map.put("page", pageNo - 1);
        map.put("size", pageSize);
        JSONArray array = new JSONArray();
        JSONObject searchCondition = new JSONObject();
        searchCondition.put("name", "isDeleted");
        searchCondition.put("op", "=");
        searchCondition.put("value", BooleanEnum.FALSE.value);
        array.add(searchCondition);
        map.put("searchCondition", array.toJSONString());
        String param = JSON.toJSONString(map);
        logger.info("-------------getBuServiceResource START -------------");
        logger.info("url:{}, param:{}", resourceUrl, param);
        String response = HttpUtil.postForm(resourceUrl, param, null, null);
        if (null == response) {
            logger.info("get bu service fail,pageNo:{},pageSize:{}", pageNo, pageSize);
            httpResult.setCode(ResponseStatus.FAIL_CODE);
        } else {
            logger.info("get app resource success,pageNo:{},pageSize:{}", pageNo, pageSize);
            JSONObject parse = JSONObject.parseObject(response);
            httpResult.setCode(ResponseStatus.SUCCESS_CODE);
            httpResult.setPages(parse.getInteger("totalPages"));
            httpResult.setTotals(parse.getInteger("totalElements"));
            httpResult.setRecords(parse.getJSONArray("content"));
        }
        logger.info("-------------getBuServiceResource END -------------");
        return httpResult;
    }

    /**
     * 从总线分页查询应用资源
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    private HttpResult getAppResource(Integer pageNo, Integer pageSize) {
        String url = resourceConfig.getResourceUrl();
        HttpResult httpResult = new HttpResult();
        Map<String, Object> map = new HashMap<>();
        //应用目录从0页开始查
        map.put("page", pageNo - 1);
        map.put("size", pageSize);
        String param = JSON.toJSONString(map);
        String busSraId = resourceConfig.getBusSraId();
        String busSreIdcard = UserContextUtils.getCurrentUser().getIdcard();
        String busServiceId = resourceConfig.getAppResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getAppResource().getBusServiceFunc();

        BusSendReqVO busSendReqVO = BusSendReqVO.builder()
                .busSraId(busSraId)
                .busServiceId(busServiceId)
                .busServiceFunc(busServiceFunc)
                .busSreIdcard(busSreIdcard)
                .build();

        List<Header> headerList = getHeaders(busSendReqVO);
        logger.info("-------------getAppResource START -------------");
        logger.info("getAppResource url:{}", url);
        String response = HttpUtil.postForm(url, param, headerList, null);
        if (null == response) {
            logger.info("get app resource fail,pageNo:{},pageSize:{}", pageNo, pageSize);
            httpResult.setCode(ResponseStatus.FAIL_CODE);
        } else {
            logger.info("get app resource success,pageNo:{},pageSize:{}", pageNo, pageSize);
            JSONObject parse = JSONObject.parseObject(response);
            httpResult.setCode(ResponseStatus.SUCCESS_CODE);
            httpResult.setPages(parse.getInteger("totalPages"));
            httpResult.setTotals(parse.getInteger("total"));
            httpResult.setRecords(parse.getJSONArray("content"));
        }
        logger.info("-------------getAppResource END -------------");
        return httpResult;
    }

    /**
     * 总线调用设置请求头
     *
     * @param busSendReqVO 请求参数
     * @return Header
     */
    private List<Header> getHeaders(BusSendReqVO busSendReqVO) {
        logger.info("getHeaders >>> {}", busSendReqVO);

        List<Header> heardList = new ArrayList<>();

        Header busSraIdHeader = new BasicHeader("bus_sra_id", busSendReqVO.getBusSraId());
        Header serviceIdHeader = new BasicHeader("bus_service_id", busSendReqVO.getBusServiceId());
        Header busServiceFuncHeader = new BasicHeader("bus_service_func", busSendReqVO.getBusServiceFunc());
        Header busSreIdHeader = new BasicHeader("bus_sre_idcard", busSendReqVO.getBusSreIdcard());

        heardList.add(busSraIdHeader);
        heardList.add(busSreIdHeader);
        heardList.add(serviceIdHeader);
        heardList.add(busServiceFuncHeader);

        String voUserToken = busSendReqVO.getUserToken();
        String voAppToken = busSendReqVO.getAppToken();

        if (StrUtil.isBlank(voUserToken) && StrUtil.isBlank(voAppToken)) {

            RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
            String userToken = "";
            String appToken = "";
            if (requestAttributes != null) {
                ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) requestAttributes;
                HttpServletRequest request = servletRequestAttributes.getRequest();
                userToken = request.getHeader(Constants.USER_TOKEN);
                appToken = request.getHeader(Constants.APP_TOKEN);
            }

            logger.info("getHeaders >>> request userToken:{},appToken:{}", userToken, appToken);
            if (StrUtil.isNotBlank(userToken)) {
                Header busSreTokenHeader = new BasicHeader("bus_sre_token", userToken);
                heardList.add(busSreTokenHeader);
            }

            if (StrUtil.isNotBlank(appToken)) {
                Header busSraTokenHeader = new BasicHeader("bus_sra_token", appToken);
                heardList.add(busSraTokenHeader);
            }
        } else {
            logger.info("getHeaders >>> request voUserToken:{}, voAppToken {}", voUserToken, voAppToken);
            Header busSraTokenHeader = new BasicHeader("bus_sra_token", voAppToken);
            heardList.add(busSraTokenHeader);

            Header busSreTokenHeader = new BasicHeader("bus_sre_token", voUserToken);
            heardList.add(busSreTokenHeader);
        }

        return heardList;
    }

    /**
     * 插入资源同步表
     *
     * @param resourceEnum
     * @param records
     * @return
     */
    private List<ResourceSyncDTO> insertRemoteResourceSync(ResourceTypeEnum resourceEnum, JSONArray records) {
        List<ResourceSyncDTO> list = new ArrayList<>();
        for (Object record : records) {
            String jsonString = JSON.toJSONString(record);
            ResourceSyncDTO dto = new ResourceSyncDTO(resourceEnum.getResourceId(),
                    resourceEnum.getResourceType(), resourceEnum.getResourceProvider());
            dto.setContent(jsonString);
            JSONObject jsonObject = JSONObject.parseObject(jsonString);
            if (ResourceTypeEnum.TJ_APP.equals(resourceEnum)) {
                String appCode = jsonObject.getString("yyxtbh");
                String appName = jsonObject.getString("yyxtmc");
                dto.setAppCode(appCode);
                dto.setAppName(appName);
            } else if (ResourceTypeEnum.TJ_SERVICE.equals(resourceEnum)) {
                String serviceCode = jsonObject.getString("FWZYBSF");
                String serviceName = jsonObject.getString("FWZYMC");
                dto.setServiceCode(serviceCode);
                dto.setServiceName(serviceName);
            } else if (ResourceTypeEnum.BU_SERVICE.equals(resourceEnum)) {
                String serviceCode = jsonObject.getString("serviceId");
                String serviceName = jsonObject.getString("serviceName");
                dto.setServiceCode(serviceCode);
                dto.setServiceName(serviceName);
            }
            dto.setExtId(jsonObject.getString("id"));
            dto.setDeleted(BooleanEnum.FALSE.getValue());
            dto.setSync(BooleanEnum.FALSE.getValue());
            list.add(dto);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            syncService.batchInsertResourceSync(list);
        }
        return list;
    }

    class MenuTask implements Runnable {

        private String idcard;

        private List<String> appIds;

        public MenuTask(List<String> appIds, String idcard) {
            this.appIds = appIds;
            this.idcard = idcard;
        }

        @Override
        public void run() {
            logger.info("get menu resource start");
            ArrayList<ResourceSyncDTO> list = new ArrayList<>();
            try {
                for (String appId : appIds) {
                    String menuStr = getAppRemoteDetailById(appId, idcard);
                    AppResourceAcceptDTO resourceAcceptDTO = JSON.parseObject(menuStr, AppResourceAcceptDTO.class);
                    AppInfoAcceptDTO appInfo = resourceAcceptDTO.getTappSystem();
                    ResourceSyncDTO dto = new ResourceSyncDTO();
                    dto.setResourceId(ResourceTypeEnum.TJ_APP_MENU.getResourceId());
                    dto.setResourceType(ResourceTypeEnum.TJ_APP_MENU.getResourceType());
                    dto.setResourceProvider(ResourceTypeEnum.TJ_APP_MENU.getResourceProvider());
                    dto.setSync(BooleanEnum.FALSE.getValue());
                    dto.setDeleted(BooleanEnum.FALSE.getValue());
                    dto.setExtId(appId);
                    dto.setAppCode(appInfo.getYyxtbh());
                    dto.setAppName(appInfo.getYyxtmc());
                    dto.setContent(JSON.toJSONString(resourceAcceptDTO.getTappFunctionList()));
                    list.add(dto);
                }
                syncService.batchInsertResourceSync(list);
            } catch (Exception e) {
                logger.info("get MenuResource wrong in appIds:{}", JSON.toJSONString(appIds));
                logger.error("get MenuResource exception.", e);
            }
        }


    }

}
