package com.dragoninfo.dcuc.app.config;


import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

/**
 * @author mazq
 * @date 2021/2/19
 */
@ConfigurationProperties(prefix = "dcuc.app.resource-index")
@Component
@Data
public class DcucResourceConfig {

    /**
     * 获取资源接口地址
     */
    private String resourceUrl;

    /**
     * bu服务资源请求地址
     */
    private String buServiceUrl;

    /**
     * 每页获取数量
     */
    private Integer pageSize;

    /**
     * 是否开启kafka,订阅增量消息
     */
    private boolean kafka;

    /**
     * 监听的topic消息
     */
    private String appTopic;

    /**
     * 监听的topic消息
     */
    private String serviceTopic;

    /**
     * 总线请求头参数
     */
    private String busSraId;

    /**
     * 总线请求头参数
     */
    private String busSreIdcard;

    /**
     * 服务资源配置
     */
    private ServiceResourceConfig serviceResource;

    /**
     * 应用资源配置
     */
    private AppResourceConfig appResource;

    /**
     * 菜单资源配置
     */
    private MenuResourceConfig menuResource;

    /**
     * 增量同步应用资源
     */
    private AppModifyResourceConfig appModifyResource;

    /**
     * 增量同步服务资源
     */
    private ServiceModifyResourceConfig serviceModifyResource;

    /**
     * 数据资源配置
     */
    private DataResourceConfig dataResource;


    @Data
    public static class MenuResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         * 根据id获取应用详情
         */
        private String busServiceFunc;

        /**
         * 是否需要获取菜单数据
         */
        private boolean menuEnabled;
    }

    @Data
    public static class AppResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         */
        private String busServiceFunc;

        /**
         * 总线请求头参数
         */
        private String busSreIdcard;
    }

    @Data
    public static class ServiceResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         */
        private String busServiceFunc;
    }

    @Data
    public static class AppModifyResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         */
        private String busServiceFunc;
    }

    @Data
    public static class ServiceModifyResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         */
        private String busServiceFunc;
    }

    @Data
    public static class DataResourceConfig {

        /**
         * 总线请求头参数
         */
        private String busServiceId;
        /**
         * 总线请求头参数
         */
        private String busServiceFunc;
    }


}

