package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.business.IResourceBusiness;
import com.dragoninfo.dcuc.app.dto.*;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ResourceSync;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.enumresources.ResourceTypeEnum;
import com.dragoninfo.dcuc.app.service.IResourceService;
import com.dragoninfo.dcuc.app.service.IResourceSyncService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * @author mazq
 * @date 2021/2/26
 **/
@Slf4j
@RestController
@RequestMapping(value = "/dcuc/app/resourceFacade")
public class ResourceFacade implements IResourceFacade {

    @Autowired
    IResourceService resourceService;

    @Autowired
    IResourceSyncService syncService;

    @Autowired
    IResourceBusiness iResourceBusiness;

    @Override
    public ResourceSyncDTO insertResourceSync(ResourceSyncDTO dto) {
        ResourceSync resourceSync = syncService.insertRemoteResourceSync(dto);
        ResourceSyncDTO resourceSyncDTO = new ResourceSyncDTO();
        BeanUtils.copyProperties(resourceSync, resourceSyncDTO);
        return resourceSyncDTO;
    }

    @Override
    public void batchInsertResourceSync(List<ResourceSyncDTO> list) {
        syncService.batchInsertResourceSync(list);
    }

    @Override
    public ApplyInfo insertAppResource(AppResourceDTO dto) {
        return resourceService.insertAppResource(dto);
    }

    @Override
    public ServiceResourceDTO insertServiceResource(ServiceResourceDTO dto) {
        ServiceResource serviceResource = resourceService.insertServiceResource(dto);
        ServiceResourceDTO serviceResourceDTO = new ServiceResourceDTO();
        BeanUtils.copyProperties(serviceResource, serviceResourceDTO);
        return serviceResourceDTO;
    }

    @Override
    public List<ApplyInfo> batchInsertAppResource(List<AppResourceDTO> list) {
        return resourceService.batchInsertAppResource(list);
    }

    @Override
    public List<ServiceResource> batchInsertServiceResource(List<ServiceResourceDTO> list) {
        return resourceService.batchInsertServiceResource(list);
    }

    @Override
    public Boolean resourceSync() {
       return iResourceBusiness.resourceSync();
    }


    @Override
    public List<ResourceRequestResultDTO> getResourceRequestResults(ResourceRequestParamDTO requestParamDTO) {
        return iResourceBusiness.getResourceRequestResults(requestParamDTO);
    }

    @Override
    public Page<ServiceResourceDTO> serviceResourcePage(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        return resourceService.page(searchable);
    }

    @Override
    public ResponseStatus subAppSync() {
        List<String> list = new ArrayList<>();
        list.add(ResourceTypeEnum.TJ_APP.getResourceType());
        ResourceRequestParamDTO paramDTO = new ResourceRequestParamDTO();
        paramDTO.setListingAll(true);
        paramDTO.setMenuSync(true);
        paramDTO.setListAllResourceType(list);
        asyncResource(paramDTO);
        return ResponseStatus.success();
    }

    @Override
    public ResponseStatus subSerSync() {
        List<String> list = new ArrayList<>();
        list.add(ResourceTypeEnum.TJ_SERVICE.getResourceType());
        ResourceRequestParamDTO paramDTO = new ResourceRequestParamDTO();
        paramDTO.setListingAll(true);
        paramDTO.setListAllResourceType(list);
        asyncResource(paramDTO);
        return ResponseStatus.success();
    }

    /**
     * 进行资源同步
     * @param paramDTO
     */
    private void asyncResource(ResourceRequestParamDTO paramDTO) {
        //因为子线程中需要调用总线需要用到请求头中的信息
        //需要将requestAttributes设置到子线程中的RequestContextHolder
        //SpringWeb暂时未提供RequestContextHolder父子线程共享ThreadLocal中的数据的配置,需要手动设置
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        //开启父子线程
        //方法内部有从ThreadLocal获取信息
        //CompletableFuture默认使用ForkJoin线程池,不要使用1.8的新线程开启方法
        //如果使用线程池需要用包装类将线程包装后设置RequestContextHolder
        Runnable r = ()->{
            RequestContextHolder.setRequestAttributes(requestAttributes);
            try {
                iResourceBusiness.getResourceRequestResults(paramDTO);
                iResourceBusiness.resourceSync();
            } finally {
                RequestContextHolder.resetRequestAttributes();
            }
        };
        Thread t = new Thread(r);
        t.start();
    }


}
