package com.dragoninfo.dcuc.app.service;

import com.dragoninfo.dcuc.app.dto.ApplyInfoDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.data.domain.Page;

import java.util.List;
import java.util.Map;


/**
 * @author yica huangzqa
 * @date 2018/8/7
 */
public interface IApplyInfoService {

    /**
     * 详情
     *
     * @param id 应用ID
     * @return 应用详情
     */
    ApplyInfo applyDetail(String id);

    /**
     * 分页查询应用
     *
     * @param searchDTO 查询条件
     * @return 应用分页
     */
    Page<ApplyInfo> applyInfoPage(SearchDTO searchDTO);

    /**
     * 保存
     *
     * @param applyInfoDTO 应用信息
     * @return 应用信息
     */
    ApplyInfo saveAppIy(ApplyInfoDTO applyInfoDTO);

    /**
     * 修改应用
     *
     * @param applyInfoDTO 应用信息传输层
     * @return 应用信息
     */
    ApplyInfo updateApply(ApplyInfoDTO applyInfoDTO);

    /**
     * 删除
     *
     * @param id 应用ID
     */
    void del(String id);

    /**
     * 找到最新的排序
     *
     * @return 最新排序
     */
    String getNewSort();

    /**
     * 启动或停止
     *
     * @param id 应用id
     * @return 消息
     */
    String startOrStop(String id);

    /**
     * 排序应用
     *
     * @param id     源应用id
     * @param target 目标应用ID
     */
    void sortApp(String id, String target);

    /**
     * 根据appCode获取appId
     *
     * @param appCode 应用代码
     * @return 应用ID
     */
    String codeConvertToId(String appCode);

    /**
     * ID转换名称
     *
     * @param projectIds id
     * @return 名称
     */
    String convertIdsToNames(String projectIds);

    /**
     * 根据appCode获取应用详情
     *
     * @param appCode 应用代码
     * @return 应用信息
     */
    ApplyInfo getAppByCode(String appCode);

    /**
     * 是否重复
     *
     * @param applyInfo 应用信息
     * @return 是否重复
     */
    ResponseStatus isRepeat(ApplyInfo applyInfo);


    /**
     * 根据施工人员id查看该人员负责的应用
     *
     * @param userId 用户id
     * @return 应用列表
     */
    List<ApplyInfo> fzxmList(String userId);

    /**
     * 保存应用负责人
     *
     * @param dutyProject 应用ID
     * @param userId      用户id
     * @param mobileWork  工作手机号
     */
    void saveDutyMan(String dutyProject, String userId,
                     String mobileWork);

    /**
     * 查询应用名称
     *
     * @return 应用名称和ID
     */
    Map<String, String> findAppNameMap();

    /**
     * 人员授权-角色视图-查找权限列表
     *
     * @param searchDTO 搜索条件
     * @return 应用信息
     */
    Page<ApplyInfo> findApplyList(SearchDTO searchDTO);

    /**
     * 获取厂商所属应用个数
     *
     * @param manufacturerId 厂商ID
     * @return 个数
     */
    Integer countByManufacturerId(String manufacturerId);

    /**
     * 根据厂商ID 获取应用
     *
     * @param manufacturerId 厂商ID
     * @return 应用列表
     */
    List<ApplyInfo> getAppListByManufacturerId(String manufacturerId);

    /**
     * 获取 key 为 id，value 为 name 的 map
     *
     * @return map
     */
    Map<String, String> getAllIdNameMap();

    /**
     * 根据应用代码、应用名称获取应用
     *
     * @param appCode 应用代码
     * @param appName 应用名称
     * @return 应用
     */
    ApplyInfo getOneByCodeAndName(String appCode, String appName);

    /**
     * 获取应用信息
     *
     * @param applyName      应用名称
     * @param manufacturerId 厂商ID
     * @return 应用信息
     */
    ApplyInfo getOneByNameAndManufacturerId(String applyName, String manufacturerId);

    /**
     * 获取应用信息
     *
     * @param applyName 应用名称
     * @return 应用信息
     */
    ApplyInfo getOneByName(String applyName);

    /**
     * 根据id找到所有应用
     *
     * @param ids
     * @return
     */
    List<ApplyInfo> getAppById(List<String> ids);

    /**
     * 根据应用名称LIKE获取应用列表
     *
     * @param applyName 应用名称
     * @return 应用列表
     */
    List<ApplyInfo> getAppByAppNameLike(String applyName);

    /**
     * 根据应用url获取应用信息
     *
     * @param applyUrl
     * @return
     */
    ApplyInfo getAppByAppUrl(String applyUrl);

    /**
     * 获取机构id集合
     *
     * @param appCodes 应用code集合
     * @return
     */
    List<ApplyInfo> getIdsByCodes(List<String> appCodes);

    ApplyInfo getOne(String id);

    List<ApplyInfo> findAll();

    ResponseStatus impExcel(String fileId);

    /**
     * 根据应用状态获取应用列表
     * @param applyStatus
     * @return
     */
    List<ApplyInfo> findList(String applyStatus);

    /**
     * 根据code集合获取应用列表
     * @param appCodes
     * @return
     */
    List<ApplyInfo> getListByCodes(List<String> appCodes);

    /**
     * 条件查询-不分页
     *
     * @param searchDTO
     * @return
     */
    List<ApplyInfo> appInfoSearch(SearchDTO searchDTO);

    /**
     * 更新机构信息
     *
     * @param targetOrgCode 目标机构代码
     * @param fullName      完整名称
     * @param code          机构代码
     */
    void updateOrgInfos(String targetOrgCode, String fullName, String code);
}
