package com.dragoninfo.dcuc.app.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.dto.ApplyInfoDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragoninfo.dcuc.app.mapper.ApplyInfoMapper;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragoninfo.dcuc.app.service.IApplyOauthService;
import com.dragoninfo.dcuc.app.service.IManufacturerInfoService;
import com.dragoninfo.dcuc.auth.auth.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.auth.auth.facade.IPermissionNotificationFacade;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragoninfo.dcuc.duceap.facade.IDuceapUploadFacade;
import com.dragoninfo.dcuc.duceap.upload.dto.DocContentDTO;
import com.dragoninfo.dcuc.duceap.upload.dto.FileUploadDTO;
import com.dragoninfo.dcuc.user.entity.org.OrgInfo;
import com.dragoninfo.dcuc.user.facade.org.IOrgInfoFacade;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.entity.uploader.FileHolder;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.search.Searchable;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author huangzqa
 * @date 2020/10/27
 */
@CacheConfig(cacheNames = "DCUC-APP-APP")
@Service
public class ApplyInfoServiceImpl implements IApplyInfoService {

    @Autowired
    private ApplyInfoMapper applyInfoMapper;

    @Resource
    private IDuceapUploadFacade uploadhandler;

    @Autowired
    private IManufacturerInfoService manufacturerInfoService;

    @Autowired
    private IApplyOauthService applyOauthService;

    @Autowired
    private IDuceapUploadFacade uploadHandler;

    @Autowired
    private IPermissionNotificationFacade permissionNotificationFacade;

    @Autowired
    private IOrgInfoFacade orgInfoFacade;

    @Cacheable(key = "targetClass +#p0")
    @Override
    public ApplyInfo applyDetail(String id) {
        return applyInfoMapper.selectById(id);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public Page<ApplyInfo> applyInfoPage(SearchDTO searchDTO) {

        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSort(Sort.Direction.DESC, "SORT");
        searchable.addSort(Sort.Direction.DESC, "REGISTRATION_TIME");
        return applyInfoMapper.pagingBySearchable(searchable);
    }

    /**
     * 校验数据是否为空
     *
     * @param cell
     * @param i
     * @param row
     */
    private static void checkServiceResourceCellData(Cell cell, int i, int row) {
        if (!(cell == null || StringUtils.isEmpty(cell.getStringCellValue()))) {
            return;
        }
        switch (i) {
            case 0:
                throw new NumberFormatException("第" + (row + 1) + "行'应用系统名称'为空！");
            case 1:
                throw new NumberFormatException("第" + (row + 1) + "行'应用系统编号'为空！");
            case 2:
                throw new NumberFormatException("第" + (row + 1) + "行'应用系统英文名称'为空！");
            case 3:
                throw new NumberFormatException("第" + (row + 1) + "行'系统在用标识'为空！");
            default:
                break;
        }

    }

    /**
     * 校验列头
     *
     * @param r
     */
    private static void checkRowTitle(Row r) {
        if (!r.getCell(0).getStringCellValue().equals("应用系统名称")) {
            throw new NumberFormatException("缺少'应用系统名称'列！");
        }
        if (!r.getCell(1).getStringCellValue().equals("应用系统编号")) {
            throw new NumberFormatException("缺少'应用系统编号'列！");
        }
        if (!r.getCell(2).getStringCellValue().equals("应用系统英文名称")) {
            throw new NumberFormatException("缺少'应用系统英文名称'列！");
        }
        if (!r.getCell(3).getStringCellValue().equals("系统在用标识")) {
            throw new NumberFormatException("缺少'系统在用标识'列！");
        }
        if (!r.getCell(4).getStringCellValue().equals("上线日期")) {
            throw new NumberFormatException("缺少'上线日期'列！");
        }
        if (!r.getCell(5).getStringCellValue().equals("应用负责人")) {
            throw new NumberFormatException("缺少'应用负责人'列！");
        }
        if (!r.getCell(6).getStringCellValue().equals("应用系统事权单位代码")) {
            throw new NumberFormatException("缺少'应用系统事权单位代码'列！");
        }
        if (!r.getCell(7).getStringCellValue().equals("应用系统管理单位名称")) {
            throw new NumberFormatException("缺少'应用系统管理单位名称'列！");
        }
        if (!r.getCell(8).getStringCellValue().equals("应用系统运维单位名称")) {
            throw new NumberFormatException("缺少'应用系统运维单位名称'列！");
        }
        if (!r.getCell(9).getStringCellValue().equals("应用系统说明")) {
            throw new NumberFormatException("缺少'应用系统说明'列！");
        }
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void del(String id) {
        ApplyInfo applyInfo = applyInfoMapper.selectById(id);
        applyInfo.setDeleted(BooleanEnum.TRUE.getValue());

        applyInfoMapper.deleteById(id);
        // 进行权限通知
        permissionNotificationFacade.sendAppAppUpdate(id);
    }

    @Cacheable(key = "targetClass + methodName")
    @Override
    public String getNewSort() {
        return applyInfoMapper.getNewSort();
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String startOrStop(String id) {
        ApplyInfo applyInfo = applyInfoMapper.selectById(id);

        String applyStatus = applyInfo.getApplyStatus();

        String message;
        if (StrUtil.isNotBlank(applyStatus) && applyStatus.equals(BooleanEnum.TRUE.getValue())) {
            applyInfo.setApplyStatus(BooleanEnum.FALSE.getValue());
            message = "启动成功";
        } else {
            applyInfo.setApplyStatus(BooleanEnum.TRUE.getValue());
            message = "停止成功";
        }

        applyInfoMapper.updateById(applyInfo);
        // 进行权限通知
        permissionNotificationFacade.sendAppAppUpdate(id);
        return message;
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sortApp(String id, String target) {
        ApplyInfo applyInfo = applyInfoMapper.selectById(id);

        ApplyInfo targetApplyInfo = applyInfoMapper.selectById(target);

        String sort = applyInfo.getSort();

        String targetSort = targetApplyInfo.getSort();

        applyInfo.setSort(targetSort);
        targetApplyInfo.setSort(sort);

        applyInfoMapper.updateById(applyInfo);
        applyInfoMapper.updateById(targetApplyInfo);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public String codeConvertToId(String appCode) {

        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();

        queryWrapper.select(ApplyInfo::getId)
                .eq(ApplyInfo::getApplyCode, appCode)
                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue());

        ApplyInfo applyInfo = applyInfoMapper.selectOne(queryWrapper);

        String id = "";
        if (applyInfo != null) {
            id = applyInfo.getId();
        }

        return id;
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public String convertIdsToNames(String projectIds) {

        Object[] projectIdArray = projectIds.split(StrUtil.COMMA);

        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ApplyInfo::getApplyName)
                .in(ApplyInfo::getId, projectIdArray);

        List<ApplyInfo> applyInfos = applyInfoMapper.selectList(queryWrapper);

        List<String> stringList = applyInfos.stream()
                .map(ApplyInfo::getApplyName)
                .collect(Collectors.toList());

        return LangUtil.array2String(stringList, StrUtil.COMMA);
    }

    @Cacheable(key = "targetClass +methodName+#p0")
    @Override
    public ApplyInfo getAppByCode(String appCode) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select().eq(ApplyInfo::getApplyCode, appCode)
                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue());

        return applyInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public List<ApplyInfo> getAppById(List<String> ids) {
        return applyInfoMapper.selectBatchIds(ids);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public List<ApplyInfo> getAppByAppNameLike(String applyName) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(ApplyInfo::getApplyName, applyName);
        return applyInfoMapper.selectList(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public ApplyInfo getAppByAppUrl(String applyUrl) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getApplyUrl, applyUrl);
        List<ApplyInfo> applyInfos = applyInfoMapper.selectList(queryWrapper);
        if (CollectionUtils.isNotEmpty(applyInfos)) {
            return applyInfos.get(0);
        } else {
            return null;
        }

    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public List<ApplyInfo> getIdsByCodes(List<String> appCodes) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ApplyInfo::getId, ApplyInfo::getApplyCode)
                .in(ApplyInfo::getApplyCode, appCodes);
        return applyInfoMapper.selectList(queryWrapper);
    }

    @Override
    public ApplyInfo getOne(String id) {
        return applyInfoMapper.selectById(id);
    }

    @Override
    public List<ApplyInfo> findAll() {
        return applyInfoMapper.selectList(Wrappers.emptyWrapper());
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ApplyInfo saveAppIy(ApplyInfoDTO applyInfoDTO) {
        ApplyInfo applyInfo = new ApplyInfo();
        BeanUtils.copyProperties(applyInfoDTO, applyInfo);

        List<FileHolder> file = applyInfoDTO.getFile();

        String manufacturerId = applyInfo.getManufacturerId();
        if (StringUtils.isNotBlank(manufacturerId)) {
            ManufacturerInfo manufacturerInfo = manufacturerInfoService.getOne(manufacturerId);
            if (manufacturerInfo != null) {
                String manufacturerName = manufacturerInfo.getManufacturerName();
                applyInfo.setManufacturerName(manufacturerName);
            }
        }

        String applyCode = applyInfo.getApplyCode();
        String secretKey = applyInfoDTO.getSecretKey();
        if (StrUtil.isBlank(secretKey)) {
            secretKey = getApiKeys(applyCode);
        }
        String apiKey = applyInfoDTO.getApiKey();
        if (StrUtil.isBlank(apiKey)) {
            apiKey = getApiKeys(applyCode);
        }

        applyInfo.setSecretKey(secretKey);
        applyInfo.setApiKey(apiKey);
        applyInfo.setDeleted(BooleanEnum.FALSE.getValue());

        applyInfoMapper.insert(applyInfo);
        // 保存 appId
        applyOauthService.saveOauth(applyInfo);
        String id = applyInfo.getId();
        // 保存图片
        if (null != file) {
            saveOrUpdateFile(id, file);
        }

        return applyInfo;
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ApplyInfo updateApply(ApplyInfoDTO applyInfoDTO) {
        ApplyInfo applyInfo = new ApplyInfo();
        BeanUtils.copyProperties(applyInfoDTO, applyInfo);
        List<FileHolder> file = applyInfoDTO.getFile();

        String manufacturerId = applyInfo.getManufacturerId();
        if (!StringUtils.isEmpty(manufacturerId)) {
            ManufacturerInfo manufacturerInfo = manufacturerInfoService.getOne(manufacturerId);
            if (manufacturerInfo != null) {
                String manufacturerName = manufacturerInfo.getManufacturerName();
                applyInfo.setManufacturerName(manufacturerName);
            }
        }

        String id = applyInfo.getId();
        //旧应用
        ApplyInfo oldApplyInfo = applyInfoMapper.selectById(id);
        if (null == oldApplyInfo) {
            return null;
        }
        applyInfo.setDeleted(BooleanEnum.FALSE.getValue());
        applyOauthService.updateOauth(applyInfo);
        applyInfoMapper.updateById(applyInfo);

        if (CollectionUtils.isNotEmpty(file)) {
            saveOrUpdateFile(id, file);
        }

        //发送消息
        //权限变更通知
        boolean isNotification = StringUtils.isNotEmpty(oldApplyInfo.getApplyUrl()) && (!oldApplyInfo.getApplyUrl().equals(applyInfo.getApplyUrl())
                || !oldApplyInfo.getApplyStatus().equals(applyInfo.getApplyStatus()));
        // 应用 url 当发送变更时进行通知
        if (isNotification) {
            permissionNotificationFacade.sendAppAppUpdate(id);
        }
        return applyInfo;
    }

    @CacheEvict(allEntries = true)
    @Override
    public ResponseStatus impExcel(String fileId) {
        try {
            DocContentDTO fileContent = uploadhandler.getFileContent(fileId);
            byte[] fileData = fileContent.getFileData();
            InputStream inputStream = new ByteArrayInputStream(fileData);
            List<ApplyInfo> list = loadInfo(inputStream);
            if (list.size() > 1000) {
                return ResponseStatus.fail("300", "导入数据超过1000行");
            }
            //设置事权单位id和code
            setOrgInfos(list);
            for (ApplyInfo applyInfo : list) {
                applyInfoMapper.insert(applyInfo);
            }
        } catch (IOException e) {
            return ResponseStatus.fail("300", "导入文件类型错误");
        } catch (NumberFormatException e) {
            return ResponseStatus.fail("300", e.getMessage());
        } catch (Exception e) {
            return ResponseStatus.fail("300", "导入失败");
        }
        return ResponseStatus.success();

    }

    private void setOrgInfos(List<ApplyInfo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<ApplyInfo> apps = list.stream()
                .filter(item -> StringUtils.isNotBlank(item.getOrgCode()))
                .collect(Collectors.toList());
        List<String> orgCodes = apps.stream().map(ApplyInfo::getOrgCode).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(orgCodes)) {
            List<OrgInfo> orgInfos = orgInfoFacade.getOrgsByCodes(orgCodes);
            Map<String, OrgInfo> map = orgInfos.stream().collect(Collectors.toMap(OrgInfo::getCode, item -> item, (old, last) -> last));
            for (ApplyInfo app : apps) {
                String orgCode = app.getOrgCode();
                OrgInfo orgInfo = map.get(orgCode);
                if (orgInfo != null) {
                    app.setOrgName(orgInfo.getFullName());
                }
            }
        }
    }

    @Override
    public List<ApplyInfo> appInfoSearch(SearchDTO searchDTO) {
        return applyInfoMapper.findBySearchable(Searchable.toSearchable(searchDTO));
    }

    @CacheEvict(allEntries = true)
    @Override
    public void updateOrgInfos(String targetOrgCode, String fullName, String code) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getOrgCode, targetOrgCode);

        ApplyInfo applyInfo = new ApplyInfo();
        applyInfo.setOrgName(fullName);

        if (!targetOrgCode.equals(code)) {
            applyInfo.setOrgCode(code);
        }
        applyInfoMapper.update(applyInfo, queryWrapper);
    }

    @Override
    public List<ApplyInfo> findList(String applyStatus) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = new LambdaQueryWrapper<>();
        LambdaQueryWrapper<ApplyInfo> wrapper = queryWrapper.select();
        if (StringUtils.isNotBlank(applyStatus)) {
            wrapper.eq(ApplyInfo::getApplyStatus, applyStatus);
        }
        wrapper.eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.value);
        return applyInfoMapper.selectList(wrapper);
    }


    @Override
    public ResponseStatus isRepeat(ApplyInfo applyInfo) {

        String id = applyInfo.getId();
        String applyName = applyInfo.getApplyName();
        String applyCode = applyInfo.getApplyCode();

        LambdaQueryWrapper<ApplyInfo> appNameQueryWrapper = new LambdaQueryWrapper<>();
        appNameQueryWrapper.eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue())
                .eq(ApplyInfo::getApplyName, applyName);

        if (StrUtil.isNotBlank(id)) {
            appNameQueryWrapper.ne(ApplyInfo::getId, id);
        }

        int appNameCount = applyInfoMapper.selectCount(appNameQueryWrapper);
        if (appNameCount > 0) {
            return ResponseStatus.fail("应用名称已存在");
        }

        LambdaQueryWrapper<ApplyInfo> appCodeQueryWrapper = Wrappers.lambdaQuery();
        appCodeQueryWrapper.eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue())
                .eq(ApplyInfo::getApplyCode, applyCode);

        if (StrUtil.isNotBlank(id)) {
            appCodeQueryWrapper.ne(ApplyInfo::getId, id);
        }

        int appCodeCount = applyInfoMapper.selectCount(appCodeQueryWrapper);
        if (appCodeCount > 0) {
            return ResponseStatus.fail("应用代码已存在");
        }

        return ResponseStatus.success();
    }

//    @CacheEvict(allEntries = true)
//    @Transactional(rollbackFor = Exception.class)
//    @Override
//    public void updateOrgInfos(String orgId, String orgName) {
//        applyInfoMapper.updateOrgInfo(orgId, orgName);
//    }
//
//    @Cacheable(key = "targetClass + methodName+#p0")
//    @Override
//    public List<String> getAppIdsByOrgId(String orgId) {
//
//        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
//
//        queryWrapper.eq(ApplyInfo::getOrgId, orgId)
//                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue());
//
//        List<ApplyInfo> applyInfos = applyInfoMapper.selectList(queryWrapper);
//
//        return applyInfos.stream().map(ApplyInfo::getId)
//                .collect(Collectors.toList());
//    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public List<ApplyInfo> fzxmList(String userId) {

        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();

        queryWrapper.eq(ApplyInfo::getFzr, userId)
                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.getValue());

        return applyInfoMapper.selectList(queryWrapper);
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveDutyMan(String dutyProject, String userId, String mobileWork) {
        Object[] projectIdArray = dutyProject.split(StrUtil.COMMA);

        LambdaUpdateWrapper<ApplyInfo> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.set(ApplyInfo::getFzr, userId)
                .set(ApplyInfo::getFzrPhone, mobileWork)
                .in(ApplyInfo::getId, projectIdArray);

        applyInfoMapper.update(new ApplyInfo(), updateWrapper);
    }

    @Cacheable(key = "targetClass + methodName")
    @Override
    public Map<String, String> findAppNameMap() {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ApplyInfo::getId, ApplyInfo::getApplyName)
                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.value);
        List<ApplyInfo> applyInfoList = applyInfoMapper.selectList(queryWrapper);

        Map<String, String> stringStringMap = new HashMap<>(applyInfoList.size());
        for (ApplyInfo applyInfo : applyInfoList) {
            stringStringMap.put(applyInfo.getId(), applyInfo.getApplyName());
        }
        return stringStringMap;
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public Page<ApplyInfo> findApplyList(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.removeSearchFilter("userBusiness_eq");
        searchable.addSort(Sort.Direction.DESC, "sort");
        return applyInfoMapper.pagingBySearchable(searchable);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public Integer countByManufacturerId(String manufacturerId) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getManufacturerId, manufacturerId)
                .eq(ApplyInfo::getDeleted, BooleanEnum.FALSE.value);

        return applyInfoMapper.selectCount(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public List<ApplyInfo> getAppListByManufacturerId(String manufacturerId) {

        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();

        queryWrapper.eq(ApplyInfo::getManufacturerId, manufacturerId);

        return applyInfoMapper.selectList(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName")
    @Override
    public Map<String, String> getAllIdNameMap() {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ApplyInfo::getId, ApplyInfo::getApplyName);

        List<ApplyInfo> applyInfoList = applyInfoMapper.selectList(queryWrapper);

        return applyInfoList.stream()
                .collect(Collectors.toMap(ApplyInfo::getId, ApplyInfo::getApplyName));
    }

    @Cacheable(key = "targetClass + methodName+#p0+#p1")
    @Override
    public ApplyInfo getOneByCodeAndName(String appCode, String appName) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getApplyCode, appCode)
                .eq(ApplyInfo::getApplyName, appName);
        return applyInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName+#p0 +#p1")
    @Override
    public ApplyInfo getOneByNameAndManufacturerId(String applyName, String manufacturerId) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getApplyName, applyName)
                .eq(ApplyInfo::getManufacturerId, manufacturerId);

        return applyInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass + methodName+#p0")
    @Override
    public ApplyInfo getOneByName(String applyName) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ApplyInfo::getApplyName, applyName);

        return applyInfoMapper.selectOne(queryWrapper);
    }

    /**
     * 生成SecretKey或者ApiKey
     *
     * @param value 应用代码
     * @return 生成随机码
     */
    private String getApiKeys(String value) {
        StringBuilder num = new StringBuilder();
        for (int i = 0; i < 8; i++) {
            int random = (int) (Math.random() * 10);
            num.append(random);
        }
        return value + num;
    }

    /**
     * 保存或者更新文件
     *
     * @param busiId         业务ID
     * @param fileHolderList 文件
     */
    private void saveOrUpdateFile(String busiId, List<FileHolder> fileHolderList) {
        if (CollectionUtil.isEmpty(fileHolderList)) {
            return;
        }
        FileHolder[] fileHolders = fileHolderList.toArray(new FileHolder[]{});

        String busiType = "";
        FileUploadDTO fileUploadDTO = new FileUploadDTO(busiId, busiType, fileHolders);
        uploadHandler.saveOrUpdate(fileUploadDTO);
    }

    @Override
    public List<ApplyInfo> getListByCodes(List<String> appCodes) {
        LambdaQueryWrapper<ApplyInfo> queryWrapper = new LambdaQueryWrapper<>();
        LambdaQueryWrapper<ApplyInfo> wrapper = queryWrapper.select().in(ApplyInfo::getApplyCode, appCodes);
        return applyInfoMapper.selectList(wrapper);
    }

    private List<ApplyInfo> loadInfo(InputStream inputStream) throws IOException, ParseException {
        //根据指定的文件输入流导入Excel从而产生Workbook对象
        Workbook wb0 = new HSSFWorkbook(inputStream);
        //获取Excel文档中的第一个表单
        Sheet sht0 = wb0.getSheetAt(0);
        List<ApplyInfo> list = new ArrayList<>();
        List<String> codeList = new ArrayList<>();
        for (Row r : sht0) {
            if (r.getRowNum() == 0) {
                checkRowTitle(r);
                continue;
            } else {
                //循环每行的每一列
                ApplyInfo entity = new ApplyInfo();
                boolean isAllEmpty = true;
                for (int i = 0; i < 10; i++) {
                    Cell cell = r.getCell(i);
                    if (cell != null) {
                        r.getCell(i).setCellType(Cell.CELL_TYPE_STRING);
                    }
                    if (cell != null && StringUtils.isNotEmpty(cell.getStringCellValue())) {
                        isAllEmpty = false;
                        break;
                    }
                }
                if (!isAllEmpty) {
                    for (int i = 0; i < 10; i++) {
                        Cell cell = r.getCell(i);
                        if (cell != null) {
                            r.getCell(i).setCellType(Cell.CELL_TYPE_STRING);
                        }
                        checkServiceResourceCellData(cell, i, r.getRowNum());
                        checkServiceResourceCellDataValid(cell, i, r.getRowNum(), entity, codeList);
                    }
                    entity.setDeleted(YesNotEnum.NO.getValue());
                    list.add(entity);
                }


            }

        }

        return list;
    }

    /**
     * 校验参数和设置参数
     *
     * @param cell
     * @param num
     * @param row
     * @param resource
     */
    private void checkServiceResourceCellDataValid(Cell cell, int num, int row, ApplyInfo resource, List<String> codeList) {
        if (cell == null) {
            return;
        }
        String value = cell.getStringCellValue();
        switch (num) {
            case 0:
                if (value.length() > 100) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统名称'长度大于100！");
                }
                resource.setApplyName(value);
                break;
            case 1:
                ApplyInfo applyInfo = getAppByCode(value);
                if (applyInfo != null) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统编号'已存在！");
                }
                if (codeList.contains(value)) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统编号'重复！");
                }
                codeList.add(value);
                resource.setApplyCode(value);
                break;
            case 2:
                if (value.length() > 255) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统英文名称'长度大于255！");
                }
                resource.setAppEnglishName(value);
                break;
            case 3:
                String activeCode = null;
                if (value.equals("停用")) {
                    activeCode = "1";
                }
                if (value.equals("启用")) {
                    activeCode = "0";
                }
                if (StringUtils.isEmpty(activeCode)) {
                    throw new NumberFormatException("第" + (row + 1) + "行'系统在用标识'不存在！");
                }
                resource.setApplyStatus(activeCode);

                break;
            case 4:

                if (StringUtils.isNotEmpty(value)) {
                    try {
                        SimpleDateFormat importFormat = new SimpleDateFormat("yyyyMMdd");
                        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Date parse = importFormat.parse(value);
                        value = format.format(parse);
                        resource.setLineTime(value);
                    } catch (ParseException e) {
                        throw new NumberFormatException("第" + (row + 1) + "行'上线时间'格式错误");
                    }
                }
                break;
            case 5:
                if (value.length() > 50) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用负责人'长度大于50！");
                }
                resource.setFzr(value);
                break;
            case 6:
                if (value.length() > 255) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统事权单位代码'长度大于255！");
                }
                resource.setOrgCode(value);
                break;
            case 7:
                if (value.length() > 255) {
                    throw new NumberFormatException("第" + (row + 1) + "行'管理单位名称'长度大于255！");
                }
                resource.setManagerOrgName(value);
                break;
            case 8:
                if (value.length() > 255) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统运维单位名称'长度大于255！");
                }
                resource.setAppOpsDepName(value);
                break;
            case 9:
                if (value.length() > 500) {
                    throw new NumberFormatException("第" + (row + 1) + "行'应用系统说明'长度大于500！");
                }
                resource.setManagerDescribe(value);
                break;
            default:
                break;

        }
    }
}
