package com.dragoninfo.dcuc.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ApplyOauth;
import com.dragoninfo.dcuc.app.mapper.ApplyOauthMapper;
import com.dragoninfo.dcuc.app.service.IApplyOauthService;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.mybaitsplus.core.services.BaseMybatisService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @author huangzqa
 * @date 2020/10/27
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ApplyOauthServiceImpl extends BaseMybatisService<ApplyOauth, String> implements IApplyOauthService {

    private ApplyOauthMapper applyOauthMapper;

    public ApplyOauthServiceImpl(ApplyOauthMapper applyOauthMapper) {
        super(applyOauthMapper);
        this.applyOauthMapper = applyOauthMapper;
    }

    @Override
    public ApplyOauth saveOauth(ApplyInfo applyInfo) {
        ApplyOauth applyOauth = new ApplyOauth();

        applyOauth.setApplyId(applyInfo.getId());
        applyOauth.setApiKey(applyInfo.getApiKey());
        applyOauth.setSecretKey(applyInfo.getSecretKey());
        applyOauth.setDeleted(BooleanEnum.FALSE.value);

        applyOauthMapper.insert(applyOauth);

        return applyOauth;
    }

    @Override
    public ApplyOauth updateOauth(ApplyInfo applyInfo) {

        String id = applyInfo.getId();

        LambdaQueryWrapper<ApplyOauth> queryWrapper = new LambdaQueryWrapper<>();

        queryWrapper.eq(ApplyOauth::getApplyId, id);

        ApplyOauth applyOauth = applyOauthMapper.selectOne(queryWrapper);

        if (applyOauth == null) {
            throw new IllegalArgumentException("ApplyOauth is does not exist.");
        }

        applyOauth.setApplyId(id);
        applyOauth.setApiKey(applyInfo.getApiKey());
        applyOauth.setSecretKey(applyInfo.getSecretKey());
        applyOauth.setDeleted(BooleanEnum.FALSE.getValue());

        applyOauthMapper.updateById(applyOauth);
        return applyOauth;
    }

    @Override
    public List<ApplyOauth> applyOauthList(Map<String, Object> map) {
        return applyOauthMapper.selectByMap(map);
    }

    @Override
    public ApplyOauth getByAppId(String applyId) {
        LambdaQueryWrapper<ApplyOauth> queryWrapper = new LambdaQueryWrapper<>();
        LambdaQueryWrapper<ApplyOauth> wrapper = queryWrapper.select()
                .eq(ApplyOauth::getApplyId, applyId);
        return applyOauthMapper.selectOne(wrapper);
    }
}
