package com.dragoninfo.dcuc.app.service.impl;

import com.dragoninfo.dcuc.app.business.IResourceBusiness;
import com.dragoninfo.dcuc.app.dto.dataresource.ResourceClassify;
import com.dragoninfo.dcuc.app.dto.dataresource.ResourceType;
import com.dragoninfo.dcuc.app.service.IDataResourceService;
import com.google.common.base.Strings;
import com.google.common.reflect.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lidr on 2021/4/14
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class DataResourceServiceImpl implements IDataResourceService {

    Logger logger = LoggerFactory.getLogger(DataResourceServiceImpl.class);

    @Autowired
    private IResourceBusiness resourceBusiness;

    @Override
    public List<ResourceClassify> getDataLevelList() {
        Map param = getParamMap(ResourceType.builder().typeCode("DATA_LEVEL").build());
        return resourceBusiness.getDataResource(param, new TypeToken<List<ResourceClassify>>() {
        }.getType());
    }

    @Override
    public List<ResourceClassify> getSecurityLevelList() {
        Map param = getParamMap(ResourceType.builder().typeCode("DATA_SECURITY_LEVEL").build());
        return resourceBusiness.getDataResource(param, new TypeToken<List<ResourceClassify>>() {
        }.getType());
    }

    @Override
    public List<ResourceClassify> getDataClassifyList() {
        Map param = getParamMap(true,
                ResourceType.builder().typeCode("RESOURCE_SJZZYJFL").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZZEJFL").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZYBQ1").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZYBQ2").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZYBQ3").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZYBQ4").build(),
                ResourceType.builder().typeCode("RESOURCE_SJZYBQ5").build()
        );
        return resourceBusiness.getDataResource(param, new TypeToken<List<ResourceClassify>>() {
        }.getType());
    }

    @Override
    public List<ResourceClassify> getFieldClassifyList() {
        Map param = getParamMap(true,
                ResourceType.builder().typeCode("LEVEL_1_COLUMN_CLASSIFY").build(),
                ResourceType.builder().typeCode("LEVEL_2_COLUMN_CLASSIFY").build()
        );
        return resourceBusiness.getDataResource(param, new TypeToken<List<ResourceClassify>>() {
        }.getType());
    }

    private Map getParamMap(ResourceType... typeCodes) {
        return getParamMap(false, typeCodes);
    }

    private Map getParamMap(boolean isBuildTree, ResourceType... typeCodes) {
        Map paramMap = new HashMap();
        List<Map> mapList = new ArrayList<>();

        for (ResourceType typeCode : typeCodes) {
            Map typeMap = new HashMap();

            typeMap.put("typeCode", typeCode.getTypeCode());
            if (Strings.isNullOrEmpty(typeCode.getParentCode())) {
                typeMap.put("parentCode", typeCode.getParentCode());
            }
            mapList.add(typeMap);
        }

        paramMap.put("resourceTypes", mapList);
        if (isBuildTree) {
            paramMap.put("isBuildTree", isBuildTree);
        }
        return paramMap;
    }
}
