package com.dragoninfo.dcuc.app.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.TypeReference;
import com.dragoninfo.dcuc.app.config.DcucResourceConfig;
import com.dragoninfo.dcuc.app.cons.ServiceStatusCons;
import com.dragoninfo.dcuc.app.dto.*;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ResourceSync;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.enumresources.ResourceTypeEnum;
import com.dragoninfo.dcuc.app.mapper.ServiceResourceMapper;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragoninfo.dcuc.app.service.IResourceService;
import com.dragoninfo.dcuc.app.strategy.ResourceOperateStrategy;
import com.dragoninfo.dcuc.auth.auth.entity.AppFunInfo;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.user.entity.org.OrgInfo;
import com.dragoninfo.dcuc.user.facade.org.IOrgInfoFacade;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.entity.page.PageRequest;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.mybaitsplus.core.services.BaseMybatisService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author mazq
 * @Description
 * @create 2020-11-25 14:21
 */

@Service
public class ResourceServiceImpl extends BaseMybatisService<ServiceResource,String> implements IResourceService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceServiceImpl.class);

    private final ServiceResourceMapper serviceResourceMapper;

    @Autowired
    private List<ResourceOperateStrategy> strategies;

    @Autowired
    private IOrgInfoFacade orgInfoFacade;

    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;

    @Autowired
    private DcucResourceConfig resourceConfig;

    @Autowired
    private IApplyInfoService applyInfoService;

    public ResourceServiceImpl(ServiceResourceMapper serviceResourceMapper) {
        super(serviceResourceMapper);
        this.serviceResourceMapper = serviceResourceMapper;
    }


    @Override
    public Page<ServiceResourceDTO> page(Searchable searchable) {
        Page<ServiceResource> page = serviceResourceMapper.pagingBySearchable(searchable);
        List<ServiceResourceDTO> dtos = new ArrayList<>();
        List<ServiceResource> content = page.getContent();
        for (ServiceResource serviceResource : content) {
            ServiceResourceDTO dto = new ServiceResourceDTO();
            BeanUtils.copyProperties(serviceResource, dto);
            dtos.add(dto);
        }
        long totalElements = page.getTotalElements();
        Pageable newPageable = new PageRequest(page.getNumber(), page.getSize());
        return new PageImpl<>(dtos, newPageable, totalElements);
    }

    @Override
    public ApplyInfo insertAppResource(AppResourceDTO dto) {
        ApplyInfo applyInfo = (ApplyInfo) insertResource(dto);
        Map<String,String> map = new HashMap<>();
        map.put(applyInfo.getApplyCode(),applyInfo.getId());
        batchInsertMenu(dto.getMenusResources(),map);
        return applyInfo;
    }

    @Override
    public ServiceResource insertServiceResource(ServiceResourceDTO dto) {
        return (ServiceResource) insertResource(dto);
    }

    @Override
    public List<ApplyInfo> batchInsertAppResource(List<AppResourceDTO> list) {
        List<ApplyInfo> apps = new ArrayList<>();
        if(CollectionUtils.isEmpty(list)){
            return apps;
        }
        setOrgsInfo(list);
        List<Object> objects = batchInsertResource(list);
        for (Object object : objects) {
            ApplyInfo applyInfo = (ApplyInfo) object;
            apps.add(applyInfo);
        }
        List<MenuResourceDTO> menuDTOList = list.stream()
                .filter(item -> item.getMenusResources() != null)
                .map(AppResourceDTO::getMenusResources)
                .flatMap(List::stream)
                .collect(Collectors.toList());
        Map<String, String> codeToIdMap = apps
                .stream()
                .collect(Collectors.toMap(ApplyInfo::getApplyCode, ApplyInfo::getId,(old,last)->last));
        batchInsertMenu(menuDTOList,codeToIdMap);
        return apps;
    }

    @Override
    public List<ServiceResource> batchInsertServiceResource(List<ServiceResourceDTO> list) {
        return batchInsertServiceResource(list, true);
    }

    /**
     * 重载方法
     * 控制是否需要设置机构相关的信息
     * @param list
     * @param needOrgInfos
     * @return
     */
    private List<ServiceResource> batchInsertServiceResource(List<ServiceResourceDTO> list , Boolean needOrgInfos) {
        List<ServiceResource> services = new ArrayList<>();
        if(CollectionUtils.isEmpty(list)){
            return services;
        }
        if(needOrgInfos) {
            setOrgsInfo(list);
        }
        List<Object> objects = batchInsertResource(list);
        for (Object object : objects) {
            ServiceResource service = (ServiceResource) object;
            services.add(service);
        }
        return services;
    }



    private List<Object> batchInsertResource(List<? extends ResourceDTO> list) {
        List<Object> objects = new ArrayList<>();
        for (ResourceDTO dto : list) {
            Object o = doInsert(dto);
            objects.add(o);
        }
        return objects;
    }

    /**
     * 根据对方提供机构编码,管理单位编码,批量获取id和name
     * @param list
     * @param <T> ResourceDTO
     * @return
     */
    private <T extends ResourceDTO> List<T> setOrgsInfo(List<T> list) {
        //根据对方提供机构编码,设置事权单位、管理单位信息
        List<String> codes = list.stream()
                .flatMap(item -> Stream.of(item.getManagerUnitCode(), item.getBuildUnitCode()))
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        Map<String, OrgInfo> orgInfoMap = getOrgInfos(codes);
        for (T resource : list) {
            String buildUnitCode = resource.getBuildUnitCode();
            String managerUnitCode = resource.getManagerUnitCode();
            setOrgIdAndName(resource, buildUnitCode, managerUnitCode, orgInfoMap);
        }
        return list;
    }

    private Map<String, OrgInfo> getOrgInfos(List<String> list) {
        if(CollectionUtils.isEmpty(list)){
            return new HashMap<>();
        }
        list = list
                .stream()
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        logger.info("get OrgIds by OrgCodes>> orgCodeList:{}",JSON.toJSONString(list));
        List<OrgInfo> orgInfos = orgInfoFacade.getOrgsByCodes(list);
        Map<String, OrgInfo> collect = orgInfos
                .stream()
                .collect(Collectors.toMap(OrgInfo::getCode, item->item,(old, last)->last));
        logger.info("get OrgIds by OrgCodes>> orgIdsList:{}",collect.keySet());
        return collect;
    }

    @Override
    public boolean resourceSync(List<ResourceSync> resourceSyncs) {
        Map<String, List<ResourceSync>> resourceMap = resourceSyncs.stream()
                .collect(Collectors.groupingBy(ResourceSync::getResourceId));
        List<ResourceSync> appList = resourceMap.get(ResourceTypeEnum.TJ_APP.getResourceId());
        List<ResourceSync> serviceList = resourceMap.get(ResourceTypeEnum.TJ_SERVICE.getResourceId());
        List<ResourceSync> menuList = resourceMap.get(ResourceTypeEnum.TJ_APP_MENU.getResourceId());
        List<ResourceSync> buServiceList = resourceMap.get(ResourceTypeEnum.BU_SERVICE.getResourceId());
        //插入应用资源
        if(null!=appList){
            List<AppResourceDTO> apps = appList.stream().filter(item->StringUtils.isNotBlank(item.getAppCode()))
                    .map(this::getAppResourceDTO).collect(Collectors.toList());
            batchInsertAppResource(apps);
        }
        //插入服务资源
        if(null!=serviceList){
            List<ServiceResourceDTO> services = serviceList.stream().filter(item->StringUtils.isNotBlank(item.getServiceCode()))
                    .map(this::getServiceResourceDTO).collect(Collectors.toList());
            batchInsertServiceResource(services);
        }
        //插入bu服务资源
        if(null != buServiceList) {
           List<ServiceResourceDTO> buServices = buServiceList.stream().filter(item->StringUtils.isNotBlank(item.getServiceCode()))
                    .map(this::getFromBuService).collect(Collectors.toList());
            batchInsertServiceResource(buServices, false);
        }

        //插入菜单目录
        if(null!=menuList) {
            List<MenuResourceDTO> menus = menuList.stream().filter(item->StringUtils.isNotBlank(item.getAppCode())).
                    map(this::getMenuResourceDTOs)
                    .flatMap(Collection::stream)
                    .collect(Collectors.toList());
            batchInsertMenu(menus);
        }
        return true;
    }


    /**
     * 批量插入应用的菜单数据
     * @param menus MenuResourceDTO 菜单DTOs
     */
    private void batchInsertMenu(List<MenuResourceDTO> menus) {
        if(CollectionUtils.isEmpty(menus)){
            return;
        }
        List<String> appCodes = menus.stream().map(MenuResourceDTO::getAppCode).collect(Collectors.toList());
        List<ApplyInfo> applyInfos = applyInfoService.getIdsByCodes(appCodes);
        Map<String, String> codeToIdMap = applyInfos
                .stream()
                .collect(Collectors.toMap(ApplyInfo::getApplyCode, ApplyInfo::getId,(old,last)->last));
        batchInsertMenu(menus,codeToIdMap);
    }


    /**
     * 批量插入应用的菜单数据
     * @param menus MenuResourceDTO 菜单DTOs
     * @param codeToIdMap appCode-appId 键值对Map
     */
    private void batchInsertMenu(List<MenuResourceDTO> menus, Map<String, String> codeToIdMap) {
        if(!resourceConfig.getMenuResource().isMenuEnabled()){
            return;
        }
        if(CollectionUtils.isEmpty(menus)){
            return;
        }
        List<AppFunInfo> funList = new ArrayList<>();
        for (MenuResourceDTO dto : menus) {
            AppFunInfo appFunInfo = new AppFunInfo();
            appFunInfo.setName(dto.getFunName());
            appFunInfo.setCode(dto.getFunCode());
            appFunInfo.setParentId(dto.getPFunCode());
            appFunInfo.setIsActive(dto.getStatus());
            if(null != codeToIdMap){
                String appId = codeToIdMap.get(dto.getAppCode());
                if(StringUtils.isBlank(appId)){
                    logger.info("menu resource appId is null");
                }
                appFunInfo.setAppId(appId);
            }
            appFunInfo.setDesignSecureType(dto.getDesignSecureType());
            appFunInfo.setFunctionUsageScenarios(dto.getFunctionUsageScenarios());
            funList.add(appFunInfo);
        }
        if(CollectionUtils.isNotEmpty(funList)){
            iAppFunInfoFacade.batchSave(funList);
        }
    }

    /**
     * 将资源同步对象转为菜单DTO
     * @param resourceSync ResourceSync 资源同步数据
     * @return
     */
    private List<MenuResourceDTO> getMenuResourceDTOs(ResourceSync resourceSync) {
        String appCode = resourceSync.getAppCode();
        String content = resourceSync.getContent();
        List<MenuResourceAcceptDTO> acceptDTOS = JSON.parseObject(content, new TypeReference<List<MenuResourceAcceptDTO>>() {{
        }});
        return convertToMenuDTOs(appCode, acceptDTOS);
    }

    public static List<MenuResourceDTO> convertToMenuDTOs(String appCode, List<MenuResourceAcceptDTO> acceptDTOS) {
        List<MenuResourceDTO> list = new ArrayList<>();
        for (MenuResourceAcceptDTO acceptDTO : acceptDTOS) {
            MenuResourceDTO dto = new MenuResourceDTO();
            dto.setFunCode(acceptDTO.getGnbh());
            dto.setPFunCode(acceptDTO.getSjgnbh());
            dto.setFunName(acceptDTO.getGnmc());
            dto.setStatus(acceptDTO.getZyzt());
            dto.setAppCode(appCode);
            list.add(dto);
        }
        return list;
    }

    /**
     * 将bu服务资源同步对象转为服务资源DTO
     * @param resourceSync
     * @return
     */
    private ServiceResourceDTO getFromBuService(ResourceSync resourceSync) {
        String content = resourceSync.getContent();
        BuServiceAcceptDTO acceptDTO = JSON.parseObject(content, BuServiceAcceptDTO.class);
        ServiceResourceDTO dto = new ServiceResourceDTO();
        ResourceTypeEnum resourceEnum = ResourceTypeEnum.BU_SERVICE;
        dto.setResourceId(resourceEnum.getResourceId());
        dto.setResourceType(resourceEnum.getResourceType());
        dto.setResourceProvider(resourceEnum.getResourceProvider());
        dto.setServiceCode(acceptDTO.getServiceId());
        dto.setServiceName(acceptDTO.getServiceName());
        dto.setServiceType(acceptDTO.getServiceType());
        dto.setBuildUnit(acceptDTO.getServiceManageDeptName());
        dto.setBuildUnitCode(acceptDTO.getServiceManageDept());
        dto.setManagerUnit(acceptDTO.getServiceManageDeptName());
        dto.setManagerUnitCode(acceptDTO.getServiceManageDept());
        String isDeleted = acceptDTO.getIsDeleted();
        dto.setDeleted(isDeleted);
        if(BooleanEnum.FALSE.value.equals(isDeleted)) {
            dto.setServiceStatus(ServiceStatusCons.STATUS_START);
        } else {
            dto.setServiceStatus(ServiceStatusCons.STATUS_DELETED);
        }
        return dto;
    }

    /**
     * 将资源同步对象转为服务资源DTO
     * @param resourceSync ResourceSync 资源同步数据
     * @return
     */
    private ServiceResourceDTO getServiceResourceDTO(ResourceSync resourceSync) {
        String content = resourceSync.getContent();
        ServiceResourceAcceptDTO acceptDTO = JSON.parseObject(content,ServiceResourceAcceptDTO.class);
        ServiceResourceDTO dto = convertToServiceDTO(acceptDTO);
        return dto;
    }

    public static ServiceResourceDTO convertToServiceDTO(ServiceResourceAcceptDTO acceptDTO) {
        ServiceResourceDTO dto = new ServiceResourceDTO();
        ResourceTypeEnum resourceEnum = ResourceTypeEnum.TJ_SERVICE;
        dto.setResourceId(resourceEnum.getResourceId());
        dto.setResourceType(resourceEnum.getResourceType());
        dto.setResourceProvider(resourceEnum.getResourceProvider());
        dto.setServiceCode(acceptDTO.getFWZYBSF());
        dto.setServiceName(acceptDTO.getFWZYMC());
        dto.setServiceType(acceptDTO.getFWLXDM());
        dto.setBuildUnitCode(acceptDTO.getFWZYSQDW_JGDM());
        dto.setManagerUnitCode(acceptDTO.getXXCZDW_GAJGJGDM());
        dto.setServiceUrl(acceptDTO.getFWFWDZ());
        String fwzyztdm = acceptDTO.getFWZYZTDM();
        dto.setServiceStatus(fwzyztdm);
        dto.setDeleted(BooleanEnum.FALSE.getValue());
        if (ServiceStatusCons.STATUS_DELETED.equals(fwzyztdm)) {
            dto.setDeleted(BooleanEnum.TRUE.getValue());
        }
        return dto;
    }

    /**
     * 将资源同步对象转为应用资源DTO
     * @param resourceSync ResourceSync 资源同步数据
     * @return
     */
    private AppResourceDTO getAppResourceDTO(ResourceSync resourceSync) {
        String content = resourceSync.getContent();
        AppInfoAcceptDTO acceptDTO = JSON.parseObject(content, AppInfoAcceptDTO.class);
        AppResourceDTO dto = convertToAppDTO(acceptDTO);
        return dto;
    }

    public static AppResourceDTO convertToAppDTO(AppInfoAcceptDTO acceptDTO) {
        ResourceTypeEnum resourceEnum = ResourceTypeEnum.TJ_APP;
        AppResourceDTO dto = new AppResourceDTO();
        dto.setResourceId(resourceEnum.getResourceId());
        dto.setResourceType(resourceEnum.getResourceType());
        dto.setResourceProvider(resourceEnum.getResourceProvider());
        dto.setAppCode(acceptDTO.getYyxtbh());
        dto.setAppName(acceptDTO.getYyxtmc());
        //应用状态和我们自己的系统标识转换
        String xtzybs = acceptDTO.getXtzybs();
        if (StringUtils.isNotBlank(xtzybs)) {
            if (BooleanEnum.TRUE.getValue().equals(xtzybs)) {
                xtzybs = BooleanEnum.FALSE.getValue();
            } else {
                xtzybs = BooleanEnum.TRUE.getValue();
            }
        }
        dto.setAppOpsDepName(acceptDTO.getYyxtywdwmc());
        dto.setAppStatus(xtzybs);
        dto.setAppUrl(acceptDTO.getYyxtfwdz());
        dto.setBuildUnitCode(acceptDTO.getYyxtsqdwdm());
        dto.setManagerUnitCode(acceptDTO.getYyxtgldwdm());
        dto.setContractUnit(acceptDTO.getYyxtcjdwmc());
        dto.setManagerDescribe(acceptDTO.getYyxtsm());
        dto.setDeleted(acceptDTO.getXtzxbs());
        String sxrq = acceptDTO.getSxrq();
        if(!StringUtils.isEmpty(sxrq)){
            dto.setLineTime(sxrq);
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                Date parse = format.parse(sxrq);
                dto.setRegistrationTime(parse);
            }catch (Exception e){

            }
        }
        dto.setApplyCategory(acceptDTO.getYyxtfldm());
        dto.setPoliceSpecies(acceptDTO.getGajzfldm());
        dto.setAppSecret(acceptDTO.getAppsecret());
        return dto;
    }

    /**
     * 保存应用、服务资源
     * @param <T> ResourceDTO
     * @param
     * @return
     */
    private <T extends ResourceDTO> Object insertResource(T dto) {
        //根据对方提供机构编码,承建单位编码,获取orgId和manufacturerId
        setOrgInfo(dto);
        return doInsert(dto);
    }

    /**
     * 根据对方提供机构编码,管理单位单位编码,获取id和name
     * @param dto ResourceDTO的子类
     * @param <T>
     * @return
     */
    private <T extends ResourceDTO> T setOrgInfo(T dto) {
        String buildUnitCode = dto.getBuildUnitCode();
        String managerUnitCode = dto.getManagerUnitCode();
        logger.info("get OrgInfo by OrgCode>>orgCode:{}, managerUnitCode:{}",buildUnitCode,managerUnitCode);
        List<String> list = new ArrayList<>();
        list.add(buildUnitCode);
        list.add(managerUnitCode);
        Map<String, OrgInfo> orgInfoMap = getOrgInfos(list);
        setOrgIdAndName(dto, buildUnitCode, managerUnitCode, orgInfoMap);
        return dto;
    }

    private <T extends ResourceDTO> void setOrgIdAndName(T dto, String buildUnitCode, String managerUnitCode, Map<String, OrgInfo> orgInfoMap) {
        if (StringUtils.isNotBlank(buildUnitCode)) {
            OrgInfo buildOrg = orgInfoMap.get(buildUnitCode);
            if (null != buildOrg) {
                dto.setBuildUnit(buildOrg.getFullName());
            }
        }
        if (StringUtils.isNotBlank(buildUnitCode)) {
            OrgInfo manageOrg = orgInfoMap.get(managerUnitCode);
            if (null != manageOrg) {
                dto.setManagerUnit(manageOrg.getFullName());
            }
        }
    }

    /**
     * 根据各自的策略保存资源数据
     * @param dto ResourceDTO
     * @return
     */
    private <T extends ResourceDTO> Object doInsert(T dto) {
        Object obj;
        String resourceType = dto.getResourceId();
        ResourceOperateStrategy operateStrategy = null;
        for (ResourceOperateStrategy strategy : strategies) {
            if (strategy.canHandle(resourceType)) {
                operateStrategy = strategy;
                break;
            }
        }
        if (operateStrategy == null) {
            throw new ApplicationException("has no strategy");
        }
        obj = operateStrategy.insertRemoteSource(dto);
        return obj;
    }

}
