package com.dragoninfo.dcuc.app.service.impl;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.dto.ResourceSyncDTO;
import com.dragoninfo.dcuc.app.entity.ResourceSync;
import com.dragoninfo.dcuc.app.mapper.ResourceSyncMapper;
import com.dragoninfo.dcuc.app.service.IResourceSyncService;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.mybaitsplus.core.services.BaseMybatisService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author mazq
 * @date 2020/11/26
 */
@Slf4j
@Service
public class ResourceSyncServiceImpl extends BaseMybatisService<ResourceSync,String> implements IResourceSyncService {

    private final ResourceSyncMapper resourceSyncMapper;

    public ResourceSyncServiceImpl(ResourceSyncMapper resourceSyncMapper) {
        super(resourceSyncMapper);
        this.resourceSyncMapper = resourceSyncMapper;
    }

    @Override
    public ResourceSync insertRemoteResourceSync(ResourceSyncDTO dto) {
        LambdaQueryWrapper<ResourceSync> wrapper = Wrappers.lambdaQuery();
        wrapper.select()
                .eq(ResourceSync::getResourceId,dto.getResourceId())
                .eq(ResourceSync::getDeleted,BooleanEnum.FALSE.getValue());
        String appCode = dto.getAppCode();
        String serviceCode = dto.getServiceCode();
        if(StringUtils.isBlank(appCode) && StringUtils.isBlank(serviceCode)){
            log.info("insertRemoteResourceSync appCode is null and serviceCode is null");
            return null;
        }
        if (!StringUtils.isBlank(appCode)) {
            wrapper.eq(ResourceSync::getAppCode,appCode);
        } else {
            wrapper.eq(ResourceSync::getServiceCode,serviceCode);
        }
        ResourceSync exist = resourceSyncMapper.selectOne(wrapper);
        Date date = new Date();
        ResourceSync resourceSync;
        if (null == exist) {
            resourceSync = new ResourceSync();
            BeanUtil.copyProperties(dto, resourceSync);
            resourceSync.setCreateTime(date);
            resourceSyncMapper.insert(resourceSync);
        } else {
            resourceSync = exist;
            BeanUtil.copyProperties(dto, resourceSync, "id","createTime");
            resourceSync.setUpdateTime(date);
            resourceSyncMapper.updateById(resourceSync);
        }
        return resourceSync;
    }

    @Override
    public void update(ResourceSync resourceSync) {
        resourceSync.setUpdateTime(new Date());
        resourceSyncMapper.updateById(resourceSync);
    }

    @Override
    public void batchUpdate(List<ResourceSync> resourceSyncs) {
        for (ResourceSync resourceSync : resourceSyncs) {
            resourceSyncMapper.updateById(resourceSync);
        }
    }

    @Override
    public List<ResourceSync> getNotSyncResource(String resourceId) {
        LambdaQueryWrapper<ResourceSync> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select()
                .eq(ResourceSync::getDeleted,BooleanEnum.FALSE.getValue())
                .eq(ResourceSync::getResourceId,resourceId)
                .eq(ResourceSync::getSync,BooleanEnum.FALSE.getValue());
        return resourceSyncMapper.selectList(queryWrapper);
    }

    @Override
    public void batchInsertResourceSync(List<ResourceSyncDTO> list) {
        for (ResourceSyncDTO dto : list) {
            insertRemoteResourceSync(dto);
        }
    }
}
