package com.dragoninfo.dcuc.app.service.sub;

import com.dragoninfo.dcuc.app.dto.sub.DataClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.DataClaDTO;
import com.dragoninfo.dcuc.app.entity.sub.DataCla;
import com.dragoninfo.dcuc.app.vo.DataResourceTreeVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.data.domain.Page;

import java.util.List;

/**
 * 数据资源分类业务接口
 * @author mazq
 * @date 2021/5/25
 */
public interface IDataClaService {

    /**
     * 四级分类保存
     * @param dto
     * @return
     */
    ResponseStatus dataClassifyAdd(DataClaAcceptDTO dto);

    /**
     * 更新
     * @param dto
     * @return
     */
    DataClaDTO update(DataClaDTO dto);

    /**
     * 新增
     * @param dto
     * @return
     */
    DataClaDTO save(DataClaDTO dto);

    /**
     * 根据id获取所有层级详情
     * @param id
     * @return
     */
    DataClaAcceptDTO detailAllLevel(String id);

    /**
     * 根据id批量获取
     * @param ids
     * @return
     */
    List<DataCla> getByIds(List<String> ids);

    /**
     * 根据code和层级获取详情
     * @param code
     * @param level
     * @return
     */
    DataCla getByCodeAndPath(String code, String level);

    /**
     * 根据Id删除所有层级数据
     * @param id
     * @return
     */
    Boolean deleteAllLevel(String id);

    /**
     * 四级分类更新
     * @param dto
     * @return
     */
    Boolean dataClassifyUpdate(DataClaAcceptDTO dto);

    /**
     * 分页列表查询,只查询第四级,拼接前三级内容
     * @param searchDTO
     * @return
     */
    Page<DataClaAcceptDTO> pageSearch(SearchDTO searchDTO);


    /**
     * 名称和层级模糊查询
     * @param name
     * @param level
     * @return
     */
    List<DataCla> getByNameLike(String name, String level);

    /**
     * 获取该层级数据分类列表
     * @param upId
     * @param level
     * @return
     */
    List<DataCla> getLevelList(String upId, String level);

    /**
     * 数据资源分类同步数据保存
     * @param nodes
     * @param path
     */
    void sync(List<DataResourceTreeVo> nodes, String path);

    /**
     * 查询所有列表,只查询第四级,拼接前三级内容
     * @return
     */
    List<DataClaAcceptDTO> getAll();

    /**
     * 获取所有数据
     * @return
     */
    List<DataCla> getAllDataCla();

    /**
     * 删除
     * @param id
     */
    void delete(String id);

    /**
     * 根据id获取详情
     * @param id
     * @return
     */
    DataCla getById(String id);
}
