package com.dragoninfo.dcuc.app.service.sub;

import com.dragoninfo.dcuc.app.dto.sub.FieldClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.FieldClaDTO;
import com.dragoninfo.dcuc.app.entity.sub.FieldCla;
import com.dragoninfo.dcuc.app.vo.DataResourceTreeVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.data.domain.Page;

import java.util.List;

/**
 * 字段分类业务类
 * @author mazq
 * @date 2021/5/31
 */
public interface IFieldClaService {

    /**
     * 两级字段分类保存
     * @param dto
     * @return
     */
    ResponseStatus fieldClassifyAdd(FieldClaAcceptDTO dto);

    /**
     * 字段分类保存
     * @param fieldClaDTO
     * @return
     */
    FieldClaDTO save(FieldClaDTO fieldClaDTO);

    /**
     * 两级字段分类保存
     * @param dto
     * @return
     */
    Boolean fieldClassifyUpdate(FieldClaAcceptDTO dto);

    /**
     * 获取字段分类详情
     * @param id
     * @return
     */
    FieldClaAcceptDTO detailAllLevel(String id);

    /**
     * 根据code和path查询字段分类详情
     * @param code
     * @param path
     * @return
     */
    FieldCla getByCodeAndPath(String code, String path);

    /**
     * 根据Id删除所有层级数据
     * @param id
     * @return
     */
    Boolean deleteAllLevel(String id);

    /**
     * 获取该层级字段分类列表
     * @param upId
     * @param level
     * @return
     */
    List<FieldClaDTO> getLevelList(String upId, String level);

    /**
     * 两级字段分级分页列表查询
     * @param searchDTO
     * @return
     */
    Page<FieldClaAcceptDTO> pageSearch(SearchDTO searchDTO);

    /**
     * 根据名称模糊查询
     * @param name
     * @param level
     * @return
     */
    List<FieldCla> getByNameLike(String name, String level);

    /**
     * 根据ids查询集合
     * @param ids
     * @return
     */
    List<FieldCla> getByIds(List<String> ids);

    /**
     * 字段分类数据同步
     * @param nodes
     * @param path
     */
    void sync(List<DataResourceTreeVo> nodes, String path);

    /**
     * 两级字段分级所有列表查询
     * @return
     */
    List<FieldClaAcceptDTO> getAll();

    /**
     * 获取所有字段分类
     * @return
     */
    List<FieldCla> getAllFieldCla();

    /**
     * 删除
     * @param id
     */
    void delete(String id);

    /**
     * 根据id获取详情
     * @return
     */
    FieldCla getById(String id);
}
