package com.dragoninfo.dcuc.app.service.sub.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dragoninfo.dcuc.app.cons.CommonCons;
import com.dragoninfo.dcuc.app.enumresources.DataAuthTypeCodeEnum;
import com.dragoninfo.dcuc.app.dto.sub.DataClaAcceptDTO;
import com.dragoninfo.dcuc.app.dto.sub.DataClaDTO;
import com.dragoninfo.dcuc.app.entity.sub.DataCla;
import com.dragoninfo.dcuc.app.enumresources.sub.DataClaTypeEnum;
import com.dragoninfo.dcuc.app.mapper.sub.DataClaMapper;
import com.dragoninfo.dcuc.app.service.sub.IDataClaService;
import com.dragoninfo.dcuc.app.vo.DataResourceTreeVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author mazq
 * @date 2021/5/25
 */
@Service
public class DataClaServiceImpl implements IDataClaService {

    @Autowired
    private DataClaMapper dataClaMapper;

    @Override
    public ResponseStatus dataClassifyAdd(DataClaAcceptDTO dto) {
        boolean saveFlag = false;
        String path = null;

        //一级分类保存
        DataCla fir = getByCodeAndPath(dto.getFirCode(), path);
        String firId;
        if(null == fir) {
            saveFlag = true;
            DataClaDTO dataClaDTO = new DataClaDTO(dto.getFirCode()
                    ,dto.getFirName(), DataClaTypeEnum.FIR.getLevel(), DataClaTypeEnum.FIR.getCode());
            dataClaDTO.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_CLA_FIR.value);
            DataClaDTO firDTO = save(dataClaDTO);
            firId = firDTO.getId();
        } else  {
            firId = fir.getId();
            path = firId;
        }

        //二级分类保存
        String secId;
        DataCla sec = null;
        if(!saveFlag) {
             sec = getByCodeAndPath(dto.getSecCode(), path);
        }
        if(null == sec) {
            saveFlag = true;
            DataClaDTO dataClaDTO = new DataClaDTO(dto.getSecCode()
                    ,dto.getSecName(), DataClaTypeEnum.SEC.getLevel(), DataClaTypeEnum.SEC.getCode());
            dataClaDTO.setFirId(firId);
            dataClaDTO.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_CLA_SEC.value);
            DataClaDTO secDTO = save(dataClaDTO);
            secId = secDTO.getId();
        } else {
            secId = sec.getId();
            path = path + CommonCons.PATH_SEP + secId;
        }

        //标签分类保存
        String labClaId;
        DataCla labCla = null;
        if(!saveFlag) {
            labCla = getByCodeAndPath(dto.getLabClaCode(), path);
        }
        if(null == labCla) {
            saveFlag = true;
            DataClaDTO dataClaDTO = new DataClaDTO(dto.getLabClaCode()
                    ,dto.getLabClaName(), DataClaTypeEnum.LAB_CLA.getLevel(), DataClaTypeEnum.LAB_CLA.getCode());
            dataClaDTO.setFirId(firId);
            dataClaDTO.setSecId(secId);
            dataClaDTO.setDataType(dto.getLabClaCode());
            DataClaDTO labClaDTO = save(dataClaDTO);
            labClaId = labClaDTO.getId();
        } else {
            labClaId = labCla.getId();
            path = path + CommonCons.PATH_SEP + labClaId;
        }

        DataCla lab = null;
        //标签保存
        if(!saveFlag) {
            lab = getByCodeAndPath(dto.getLabCode(), path);
        }
        if(null != lab) {
            return ResponseStatus.fail("标签代码已存在");
        }
        DataClaDTO dataClaDTO = new DataClaDTO(dto.getLabCode()
                ,dto.getLabName(), DataClaTypeEnum.LAB.getLevel(), DataClaTypeEnum.LAB.getCode());
        dataClaDTO.setRemark(dto.getRemark());
        dataClaDTO.setFirId(firId);
        dataClaDTO.setSecId(secId);
        dataClaDTO.setDataType(dto.getLabClaCode());
        dataClaDTO.setLabClaId(labClaId);
        save(dataClaDTO);
        return ResponseStatus.success();
    }

    @Override
    public DataClaDTO update(DataClaDTO dto) {
        String id = dto.getId();
        DataCla dataCla = dataClaMapper.selectById(id);
        if(dataCla == null){
            return new DataClaDTO();
        }
        BeanUtils.copyProperties(dto, dataCla,"createTime");
        dataCla.setUpdateTime(new Date());
        dataClaMapper.updateById(dataCla);
        return dto;
    }

    @Override
    public DataClaDTO save(DataClaDTO dto) {
        DataCla dataCla = new DataCla();
        BeanUtils.copyProperties(dto, dataCla);
        dataCla.setCreateTime(new Date());
        dataCla.setDeleted(BooleanEnum.FALSE.value);
        dataClaMapper.insert(dataCla);
        dto.setId(dataCla.getId());
        return dto;
    }

    @Override
    public DataClaAcceptDTO detailAllLevel(String id) {
        DataClaAcceptDTO acceptDTO = new DataClaAcceptDTO();
        DataCla dataCla = dataClaMapper.selectById(id);
        if(null == dataCla) {
            return acceptDTO;
        }
        List<String> ids = new ArrayList<>();
        String firId = dataCla.getFirId();
        String secId = dataCla.getSecId();
        String labClaId = dataCla.getLabClaId();
        ids.add(firId);
        ids.add(secId);
        ids.add(labClaId);
        List<DataCla> upDataClaList = getByIds(ids);
        //拼接成一条数据
        List<DataCla> list = new ArrayList<>();
        list.add(dataCla);
        upDataClaList.add(dataCla);
        List<DataClaAcceptDTO> tableList = convertToTableList(list, upDataClaList);
        acceptDTO = tableList.get(0);
        return acceptDTO;
    }

    @Override
    public List<DataCla> getByIds(List<String> ids) {
        if(CollectionUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<DataCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(DataCla::getId, ids);
        return dataClaMapper.selectList(queryWrapper);
    }

    @Override
    public DataCla getByCodeAndPath(String code, String path) {
        LambdaQueryWrapper<DataCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(DataCla::getCode, code);
        DataClaTypeEnum dataClaTypeEnum;
        if(StringUtils.isBlank(path)) {
            dataClaTypeEnum = DataClaTypeEnum.FIR;
        } else {
            String[] pathArr = path.split(CommonCons.PATH_SEP);
            int length = pathArr.length;
            if(CommonCons.PATH_LEVEL_FIR == length) {
                dataClaTypeEnum = DataClaTypeEnum.SEC;
                queryWrapper.eq(DataCla::getFirId,pathArr[0]);
            }else if(CommonCons.PATH_LEVEL_SEC == length) {
                dataClaTypeEnum = DataClaTypeEnum.LAB_CLA;
                queryWrapper.eq(DataCla::getFirId,pathArr[0]);
                queryWrapper.eq(DataCla::getSecId,pathArr[1]);
            }else {
                dataClaTypeEnum = DataClaTypeEnum.LAB;
                queryWrapper.eq(DataCla::getFirId,pathArr[0]);
                queryWrapper.eq(DataCla::getSecId,pathArr[1]);
                queryWrapper.eq(DataCla::getLabClaId,pathArr[2]);
            }

        }
        queryWrapper.eq(DataCla::getLevel, dataClaTypeEnum.getLevel());
        return dataClaMapper.selectOne(queryWrapper);
    }

    @Override
    public Boolean deleteAllLevel(String id) {
        DataCla dataCla = dataClaMapper.selectById(id);
        if(null == dataCla) {
            return false;
        }
        delete(dataCla.getId());
        deleteChildLevel(dataCla);
        return true;
    }

    @Override
    public Boolean dataClassifyUpdate(DataClaAcceptDTO dto) {
        String firId = dto.getFirId();
        String secId = dto.getSecId();
        String labClaId = dto.getLabClaId();
        String labId = dto.getLabId();

        if(StringUtils.isNotBlank(firId) ) {
            DataClaDTO dataClaDTO = new DataClaDTO();
            dataClaDTO.setId(firId);
            dataClaDTO.setCode(dto.getFirCode());
            dataClaDTO.setName(dto.getFirName());
            updateCodeName(dataClaDTO);
        }
        if(StringUtils.isNotBlank(secId)) {
            DataClaDTO dataClaDTO = new DataClaDTO();
            dataClaDTO.setId(secId);
            dataClaDTO.setCode(dto.getSecCode());
            dataClaDTO.setName(dto.getSecName());
            updateCodeName(dataClaDTO);
        }
        if(StringUtils.isNotBlank(labClaId)) {
            DataClaDTO dataClaDTO = new DataClaDTO();
            dataClaDTO.setId(labClaId);
            dataClaDTO.setCode(dto.getLabClaCode());
            dataClaDTO.setName(dto.getLabClaName());
            updateCodeName(dataClaDTO);
        }
        if(StringUtils.isNotBlank(labId)) {
            DataClaDTO dataClaDTO = new DataClaDTO();
            dataClaDTO.setId(labId);
            dataClaDTO.setCode(dto.getLabCode());
            dataClaDTO.setName(dto.getLabName());
            dataClaDTO.setRemark(dto.getRemark());
            updateCodeName(dataClaDTO);
        }
        return true;
    }

    @Override
    public Page<DataClaAcceptDTO> pageSearch(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("level", SearchOperator.eq,DataClaTypeEnum.LAB.getLevel());
        //一级分类名
        Condition firName = searchable.getSearchFilter("firName", SearchOperator.like);
        //查询一级分类id
        if(null != firName) {
            List<DataCla> list = getByNameLike(firName.getValue().toString(), DataClaTypeEnum.FIR.getLevel());
            if(CollectionUtils.isEmpty(list)) {
                return new PageImpl<>(new ArrayList<>(), searchable.getPage(), 0);
            }
            List<String> firIds = Optional.ofNullable(list).orElse(new ArrayList<>())
                    .stream().map(DataCla::getId).distinct().collect(Collectors.toList());
            searchable.addSearchFilter("fir_id", SearchOperator.in, firIds);
            searchable.removeSearchFilter("firName", SearchOperator.like);
        }

        //二级分类名
        Condition secName = searchable.getSearchFilter("secName", SearchOperator.like);
        if(null != secName) {
            List<DataCla> list = getByNameLike(secName.getValue().toString(), DataClaTypeEnum.SEC.getLevel());
            if(CollectionUtils.isEmpty(list)) {
                return new PageImpl<>(new ArrayList<>(), searchable.getPage(), 0);
            }
            List<String> secIds = Optional.ofNullable(list).orElse(new ArrayList<>())
                    .stream().map(DataCla::getId).distinct().collect(Collectors.toList());
            searchable.addSearchFilter("sec_id", SearchOperator.in, secIds);
            searchable.removeSearchFilter("secName", SearchOperator.like);
        }
        //标签名
        Condition labName = searchable.getSearchFilter("labName", SearchOperator.like);
        if(null != labName) {
            searchable.addSearchFilter("name", SearchOperator.like, labName.getValue().toString());
            searchable.removeSearchFilter("labName", SearchOperator.like);
        }
        Page<DataCla> page = dataClaMapper.pagingBySearchable(searchable);
        List<DataCla> content = page.getContent();
        List<DataCla> upList = getUpList(content);
        upList.addAll(content);
        List<DataClaAcceptDTO> tableList = convertToTableList(content, upList);
        return new PageImpl<>(tableList, page.getPageable(), page.getTotalElements());
    }

    @Override
    public List<DataCla> getByNameLike(String name, String level) {
        LambdaQueryWrapper<DataCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().like(DataCla::getName, name);
        if(StringUtils.isNotBlank(level)) {
            queryWrapper.eq(DataCla::getLevel, level);
        }
        return dataClaMapper.selectList(queryWrapper);
    }

    @Override
    public List<DataCla> getLevelList(String upId, String level) {
        LambdaQueryWrapper<DataCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(DataCla::getLevel, level);
        if(StringUtils.isNotBlank(upId)) {
            if(DataClaTypeEnum.SEC.getLevel().equals(level)) {
                queryWrapper.eq(DataCla::getFirId, upId);
            } else if(DataClaTypeEnum.LAB_CLA.getLevel().equals(level)) {
                queryWrapper.eq(DataCla::getSecId, upId);
            } else if(DataClaTypeEnum.LAB.getLevel().equals(level)) {
                queryWrapper.eq(DataCla::getLabClaId, upId);
            }
        }
        return dataClaMapper.selectList(queryWrapper);
    }

    /**
     * 数据分级分为四级,从一级到四级保存
     * @param nodes 这一级要保存的node
     * @param path 上级所有id组成的路径 -分隔
     */
    @Override
    public void sync(List<DataResourceTreeVo> nodes, String path) {
        if(CollectionUtils.isEmpty(nodes)) {
            return;
        }
        for (DataResourceTreeVo node : nodes) {
            String code = node.getCode();
            DataCla dataCla = getByCodeAndPath(code, path);
            String localPath = path;
            if(dataCla == null) {
                saveNodeAndChild(node, localPath, new Date());
            } else {
                updateNode(node, dataCla);
                localPath = localPath == null? dataCla.getId():path + CommonCons.PATH_SEP + dataCla.getId();
                sync(node.getChild(), localPath);
            }
        }
    }

    @Override
    public List<DataClaAcceptDTO> getAll() {
        LambdaQueryWrapper<DataCla> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(DataCla::getLevel, DataClaTypeEnum.LAB.getLevel());
        List<DataCla> list = dataClaMapper.selectList(queryWrapper);
        List<DataCla> upList = getUpList(list);
        upList.addAll(list);
        List<DataClaAcceptDTO> tableList = convertToTableList(list, upList);
        return tableList;
    }

    @Override
    public List<DataCla> getAllDataCla() {
        return dataClaMapper.selectList(null);
    }

    private List<DataCla> getUpList(List<DataCla> list) {
        List<String> firIds = list.stream().map(DataCla::getFirId).distinct().collect(Collectors.toList());
        List<String> secIds = list.stream().map(DataCla::getSecId).distinct().collect(Collectors.toList());
        List<String> labClaIds = list.stream().map(DataCla::getLabClaId).distinct().collect(Collectors.toList());
        List<String> ids = Stream.of(firIds, secIds, labClaIds).flatMap(List::stream).collect(Collectors.toList());
        return getByIds(ids);
    }

    /**
     * 更新本级树节点
     * @param node
     * @param dataCla
     */
    private void updateNode(DataResourceTreeVo node, DataCla dataCla) {
        LambdaUpdateWrapper<DataCla> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(DataCla::getName, node.getLabel())
                .set(DataCla::getUpdateTime, new Date())
                .eq(DataCla::getId, dataCla.getId());
        dataClaMapper.update(null, updateWrapper);
    }

    /**
     * 保存树节点所有子节点
     * @param node 需要保存的顶级树节点
     * @param path 上级所有id组成的路径 -分隔
     * @param date
     */
    private void saveNodeAndChild(DataResourceTreeVo node, String path, Date date) {
        DataCla dataCla = new DataCla();
        dataCla.setName(node.getLabel());
        dataCla.setCode(node.getCode());
        dataCla.setDataType(node.getDataType());
        dataCla.setDeleted(BooleanEnum.FALSE.value);
        dataCla.setCreateTime(date);
        int length = 1;
        if(StringUtils.isNotBlank(path)) {
            String[] idArr = path.split(CommonCons.PATH_SEP);
            length = idArr.length + 1;
            if(length > CommonCons.PATH_LEVEL_THIR) {
                dataCla.setLabClaId(idArr[2]);
            }
            if(length > CommonCons.PATH_LEVEL_SEC) {
                dataCla.setSecId(idArr[1]);
            }
            if(length > CommonCons.PATH_LEVEL_FIR) {
                dataCla.setFirId(idArr[0]);
            }
        }
        DataClaTypeEnum typeEnum = DataClaTypeEnum.getTypeByLevel(length);
        if(null != typeEnum) {
            dataCla.setLevel(typeEnum.getLevel());
            dataCla.setType(typeEnum.getCode());
        }
        dataClaMapper.insert(dataCla);
        path = path == null? dataCla.getId():path + CommonCons.PATH_SEP + dataCla.getId();
        List<DataResourceTreeVo> child = node.getChild();
        if(CollectionUtils.isNotEmpty(child)) {
            for (DataResourceTreeVo childNode : child) {
                saveNodeAndChild(childNode, path, date);
            }
        }

    }


    private void updateCodeName(DataClaDTO dataClaDTO) {
        LambdaUpdateWrapper<DataCla> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(DataCla::getCode, dataClaDTO.getCode())
                .set(DataCla::getName, dataClaDTO.getName());
        if(StringUtils.isNotBlank(dataClaDTO.getRemark())) {
            updateWrapper.set(DataCla::getRemark, dataClaDTO.getRemark());
        }
        updateWrapper.set(DataCla::getUpdateTime, new Date());
        updateWrapper.eq(DataCla::getId,dataClaDTO.getId());
        dataClaMapper.update(null, updateWrapper);
    }

    private void deleteChildLevel(DataCla dataCla) {
        String level = dataCla.getLevel();
        String id = dataCla.getId();
        LambdaUpdateWrapper<DataCla> updateWrapper = new LambdaUpdateWrapper<>();
        if (DataClaTypeEnum.FIR.getLevel().equals(level)) {
            updateWrapper.eq(DataCla::getFirId,id);
        } else if(DataClaTypeEnum.SEC.getLevel().equals(level)) {
            updateWrapper.eq(DataCla::getSecId,id);
        } else if(DataClaTypeEnum.LAB_CLA.getLevel().equals(level)) {
            updateWrapper.eq(DataCla::getLabClaId,id);
        } else {
            return;
        }
        updateWrapper
                .set(DataCla::getDeleted, BooleanEnum.TRUE.value)
                .set(DataCla::getDeleteUser, UserContextUtils.getCurrentUser().getId())
                .set(DataCla::getDeleteTime, new Date());
        dataClaMapper.update(null, updateWrapper);
    }

    @Override
    public void delete(String id) {
       dataClaMapper.deleteById(id);
    }

    @Override
    public DataCla getById(String id) {
        return dataClaMapper.selectById(id);
    }

    /**
     * 组装成列表
     * @param dataClaList
     * @param upList
     * @return
     */
    private List<DataClaAcceptDTO> convertToTableList(List<DataCla> dataClaList, List<DataCla> upList) {
        List<DataClaAcceptDTO> dtos = new ArrayList<>();
        if(CollectionUtils.isEmpty(dataClaList)) {
            return dtos;
        }
        Map<String, List<DataCla>> map = Optional.ofNullable(upList).orElse(new ArrayList<>())
                .stream().collect(Collectors.groupingBy(DataCla::getLevel));
        //一级分级
        List<DataCla> firList = map.get(DataClaTypeEnum.FIR.getLevel());
        Map<String, DataCla> firMap = Optional.ofNullable(firList)
                .orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(DataCla::getId, item -> item, (old, last) -> last));
        //二级分级
        List<DataCla> secList = map.get(DataClaTypeEnum.SEC.getLevel());
        Map<String, DataCla> secMap = Optional.ofNullable(secList)
                .orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(DataCla::getId, item -> item, (old, last) -> last));
        //标签分类(三级)
        List<DataCla> labClaList = map.get(DataClaTypeEnum.LAB_CLA.getLevel());
        Map<String, DataCla> labClaMap = Optional.ofNullable(labClaList)
                .orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(DataCla::getId, item -> item, (old, last) -> last));
        //组装成列表
        if (CollectionUtils.isNotEmpty(dataClaList)) {
            for (DataCla dataCla : dataClaList) {
                String firId = dataCla.getFirId() == null ? dataCla.getId(): dataCla.getFirId();
                String secId = dataCla.getSecId() == null ? dataCla.getId(): dataCla.getSecId();
                String labClaId = dataCla.getLabClaId() == null? dataCla.getId(): dataCla.getLabClaId();
                DataClaAcceptDTO dto = new DataClaAcceptDTO();
                int level = Integer.parseInt(dataCla.getLevel());
                //标签属性
                if(level >= CommonCons.PATH_LEVEL_FOUR) {
                    dto.setLabCode(dataCla.getCode());
                    dto.setLabName(dataCla.getName());
                    dto.setLabId(dataCla.getId());
                    dto.setRemark(dataCla.getRemark());
                }
                //标签分类属性
                if(level >= CommonCons.PATH_LEVEL_THIR) {
                    DataCla labCla = labClaMap.get(labClaId);
                    if(null != labCla) {
                        dto.setLabClaCode(labCla.getCode());
                        dto.setLabClaName(labCla.getName());
                        dto.setLabClaId(labCla.getId());
                    }
                }
                //二级分类属性
                if(level >= CommonCons.PATH_LEVEL_SEC) {
                    DataCla sec = secMap.get(secId);
                    if(null != sec) {
                        dto.setSecCode(sec.getCode());
                        dto.setSecName(sec.getName());
                        dto.setSecId(sec.getId());
                    }

                }
                //一级分类属性
                if(level >= CommonCons.PATH_LEVEL_FIR) {
                    DataCla fir = firMap.get(firId);
                    if(null != fir) {
                        dto.setFirCode(fir.getCode());
                        dto.setFirName(fir.getName());
                        dto.setFirId(fir.getId());
                    }
                }
                dtos.add(dto);
            }
        }
        return dtos;
    }



}
