package com.dragoninfo.dcuc.app.business;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.app.DcucAppApplication;
import com.dragoninfo.dcuc.app.business.impl.ResourceBusiness;
import com.dragoninfo.dcuc.app.config.DcucResourceConfig;
import com.dragoninfo.dcuc.app.facade.DataResourceFacade;
import com.dragoninfo.dcuc.app.vo.DataResourceTreeVo;
import com.dragoninfo.dcuc.common.http.HttpUtil;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.mq.client.model.entity.MessageInfoDTO;
import org.apache.http.Header;
import org.apache.http.message.BasicHeader;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author mazq
 * @date 2021/3/17
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = DcucAppApplication.class)
public class ResourceBusinessTest {

    @Autowired
    private ResourceBusiness resourceBusiness;

    @Autowired
    DcucResourceConfig resourceConfig;

    @Autowired
    DataResourceFacade dataResourceFacade;

    @Test
    public void receiveMessageTest() {
        String message = "{\"content\":\"{\\\"yyxtbh\\\":\\\"A-120000000000-0206\\\",\\\"sqlx\\\":\\\"A-00\\\"}\",\"createTime\":\"20210521094917\",\"infoType\":\"10000028\",\"messageId\":\"2c64278a598d4dc4b954ef9144ecc4f6\",\"receiveDepts\":\"\",\"receiveRoles\":\"\",\"receiveUsers\":\"\",\"title\":\"2021-05-21应用新增/变更通知\",\"userCode\":\"460004197411064109\",\"userDeptId\":\"\",\"userDeptName\":\"\",\"userName\":\"大数据运营服务平台\",\"userPoliceNo\":\"\",\"userSystem\":\"大数据运营服务平台\",\"userSystemCode\":\"dsjyyfwpt\"}";
        MessageInfoDTO messageInfoDTO = JSON.parseObject(message, MessageInfoDTO.class);
        String content = messageInfoDTO.getContent();
        if (StringUtils.isBlank(content)) {
            return;
        }
        JSONObject jsonObject = JSONObject.parseObject(content);
        String appCode = jsonObject.getString("yyxtbh");
        if (StringUtils.isBlank(appCode)) {
            return;
        }
        resourceBusiness.appIncrSync(appCode);

    }


    @Test
    public void getDataResource() {
        Map paramMap = new HashMap();
        Map typeMap = new HashMap();
        List<Map> mapList = new ArrayList<>();
        typeMap.put("typeCode", "DATA_LEVEL");
        mapList.add(typeMap);
        paramMap.put("resourceTypes", mapList);

        // 构建header
        String busSraId = resourceConfig.getBusSraId();
        busSraId = "Y-120000000000-0037";//原有身份没有权限 测试数据
        String busSreIdcard = resourceConfig.getBusSreIdcard();
        String busServiceId = resourceConfig.getDataResource().getBusServiceId();
        String busServiceFunc = resourceConfig.getDataResource().getBusServiceFunc();
        List<Header> heards = getHeaders(busSraId, busSreIdcard, busServiceId, busServiceFunc);
        String url = resourceConfig.getResourceUrl();
        String response = HttpUtil.postJSON(url, JSON.toJSONString(paramMap), heards, null);

        Map resultMap = (Map)JSON.parse(response);
        List<Map> resultData = (List)resultMap.get("resultData");
        List<Map> resultList = (List)resultData.get(0).get("dataList");

    }

    private List<Header> getHeaders(String busSraId, String busSreIdcard, String busServiceId, String busServiceFunc) {
        List<Header> heardList = new ArrayList<>();
        Header busSraIdHeader = new BasicHeader("bus_sra_id",busSraId);
        Header busSreIdHeader = new BasicHeader("bus_sre_idcard", busSreIdcard);
        Header serviceIdHeader = new BasicHeader("bus_service_id",busServiceId);
        Header busServiceFuncHeader = new BasicHeader("bus_service_func",busServiceFunc);
        heardList.add(busSraIdHeader);
        heardList.add(busSreIdHeader);
        heardList.add(serviceIdHeader);
        heardList.add(busServiceFuncHeader);
        return heardList;
    }


    @Test
    public void getFieldCla() {
        dataResourceFacade.fieldClaSync();
    }

}
