package com.dragoninfo.dcuc.app.service.impl;

import cn.hutool.json.JSONUtil;
import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragoninfo.dcuc.app.service.IManufacturerInfoService;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.Page;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author huangzqa
 * @date 2020/10/28
 */
@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
@Rollback()
public class ManufacturerInfoServiceImplTest {

    @Autowired
    private IManufacturerInfoService manufacturerInfoService;

    @Test
    public void manufacturerInfoPage() {
        SearchDTO searchDTO = new SearchDTO();
        searchDTO.setSize("10");
        searchDTO.setPage("1");

        Page<ManufacturerInfo> manufacturerInfos = manufacturerInfoService.manufacturerInfoPage(searchDTO);
        System.out.println(JSONUtil.toJsonStr(manufacturerInfos));
    }


    @Test
    public void manufacturerInfoDetail() {

        String id = "2c90e9e1651c465901651c47edc80000";

        ManufacturerInfo manufacturerInfo = manufacturerInfoService.manufacturerInfoDetail(id);

        Assert.assertEquals(manufacturerInfo.getId(), id);

        Assert.assertEquals(manufacturerInfo.getCode(), "2222");

    }

    @Test
    public void saveManufacturer() {
        ManufacturerInfo manufacturerInfo = new ManufacturerInfo();
        manufacturerInfo.setManufacturerName("SpringBootTest");
        manufacturerInfo.setName("test");
        manufacturerInfo.setIdcard("1111111");
        manufacturerInfo.setCode("SpringBootTest");
        manufacturerInfo.setAddress("1111111111");

        manufacturerInfoService.saveManufacturer(manufacturerInfo);

        ManufacturerInfo selectManufacturerInfo = manufacturerInfoService.detailByCode("11111");

        Assert.assertEquals(selectManufacturerInfo.getManufacturerName(), "SpringBootTest");
    }

    @Test
    public void updateManufacturer() {
        String code = "SpringBootTest";
        ManufacturerInfo manufacturerInfo = manufacturerInfoService.detailByCode(code);
        manufacturerInfo.setAddress("new Address");
        manufacturerInfoService.updateManufacturer(manufacturerInfo);

        ManufacturerInfo manufacturerInfo1 = manufacturerInfoService.detailByCode(code);

        Assert.assertEquals(manufacturerInfo1.getAddress(), "new Address");
    }

    @Test
    public void del() {
        String id = "17fc8004f948ecfd5a30745d2b67b64f";
        ManufacturerInfo one = manufacturerInfoService.findOne(id);
        Assert.assertNotNull(one);
        manufacturerInfoService.del(id);
        ManufacturerInfo one1 = manufacturerInfoService.findOne(id);
        Assert.assertNull(one1);
    }

    @Test
    public void detailByCode() {
        String code = "SpringBootTest";
        ManufacturerInfo manufacturerInfo = manufacturerInfoService.detailByCode(code);

        Assert.assertEquals(manufacturerInfo.getCode(), code);
    }
}